@@ -10,14 +10,6 @@
 #define NULL ((void*)0)
 #endif
 
-/*
- * Warning: proplist.h #defines BOOL which will clash with the
- * typedef BOOL in Xmd.h
- * proplist.h should use Bool (which is a #define in Xlib.h) instead.
- *
- */
-#include <proplist.h>
-
 
 #ifndef WMAX
 # define WMAX(a,b)	((a)>(b) ? (a) : (b))
@@ -121,13 +113,13 @@ typedef struct W_Array WMArray;
 typedef struct W_Bag WMBag;
 typedef struct W_Data WMData;
 typedef struct W_TreeNode WMTreeNode;
-typedef struct W_HashTable WMDictionary;
 typedef struct W_HashTable WMHashTable;
 typedef struct W_UserDefaults WMUserDefaults;
 typedef struct W_Notification WMNotification;
 typedef struct W_NotificationQueue WMNotificationQueue;
 typedef struct W_Host WMHost;
 typedef struct W_Connection WMConnection;
+typedef struct W_PropList WMPropList;
 
 
 
@@ -231,7 +223,7 @@ char* wfindfile(char *paths, char *file);
 
 char* wfindfileinlist(char **path_list, char *file);
 
-char* wfindfileinarray(proplist_t array, char *file);
+char* wfindfileinarray(WMPropList* array, char *file);
 
 char* wexpandpath(char *path);
 
@@ -332,22 +324,41 @@ void WMFreeHashTable(WMHashTable *table);
 
 void WMResetHashTable(WMHashTable *table);
 
+unsigned WMCountHashTable(WMHashTable *table);
+
 void* WMHashGet(WMHashTable *table, const void *key);
 
+/* Returns True if there is a value associated with <key> in the table, in
+ * which case <retKey> and <retItem> will contain the item's internal key
+ * address and the item's value respectively.
+ * If there is no value associated with <key> it will return False and in
+ * this case <retKey> and <retItem> will be undefined.
+ * Use this when you need to perform your own custom retain/release mechanism
+ * which requires access to the keys too.
+ */
+Bool WMHashGetItemAndKey(WMHashTable *table, const void *key,
+                         void **retItem, void **retKey);
+
 /* put data in table, replacing already existing data and returning
  * the old value */
 void* WMHashInsert(WMHashTable *table, const void *key, const void *data);
 
 void WMHashRemove(WMHashTable *table, const void *key);
 
-/* warning: do not manipulate the table while using these functions */
+/* warning: do not manipulate the table while using the enumerator functions */
 WMHashEnumerator WMEnumerateHashTable(WMHashTable *table);
 
 void* WMNextHashEnumeratorItem(WMHashEnumerator *enumerator);
 
 void* WMNextHashEnumeratorKey(WMHashEnumerator *enumerator);
 
-unsigned WMCountHashTable(WMHashTable *table);
+/* Returns True if there is a next element, in which case key and item
+ * will contain the next element's key and value respectively.
+ * If there is no next element available it will return False and in this
+ * case key and item will be undefined.
+ */
+Bool WMNextHashEnumeratorItemAndKey(WMHashEnumerator *enumerator,
+                                    void **item, void **key);
 
 
 
@@ -362,7 +373,7 @@ extern const WMHashTableCallbacks WMStringHashCallbacks;
  * and freed with wfree() */
 
 extern const WMHashTableCallbacks WMStringPointerHashCallbacks;
-/* keys are strings, bug they are not copied */
+/* keys are strings, but they are not copied */
 
 
 /*......................................................................*/
@@ -672,17 +683,6 @@ WMTreeNode* WMFindInTree(WMTreeNode *aTree, WMMatchDataProc *match, void *cdata)
 #define WMGetFirstInTree(aTree, cdata) WMFindInTree(atree, NULL, cdata)
 
 
-/*--------------------------------------------------------------------------*/
-
-/* Dictionaries */
-/*
-WMDictionary* WMCreateDictionaryFromElements(void *key, void *value, ...);
-
-#define WMGetDictionaryEntryForKey(dict, key) WMHashGet(dict, key)
-
-WMArray* WMGetAllDictionaryKeys(WMDictionary *dPtr);
-*/
-
 /*--------------------------------------------------------------------------*/
 
 
@@ -729,6 +729,108 @@ void WMEnqueueCoalesceNotification(WMNotificationQueue *queue,
 				   unsigned coalesceMask);
 
 
+/*......................................................................*/
+
+/* Property Lists handling */
+
+void WMPLSetCaseSensitive(Bool caseSensitive);
+
+WMPropList* WMCreatePLString(char *str);
+
+WMPropList* WMCreatePLData(WMData *data);
+
+WMPropList* WMCreatePLDataWithBytes(unsigned char *bytes, unsigned int length);
+
+WMPropList* WMCreatePLDataWithBytesNoCopy(unsigned char *bytes,
+                                          unsigned int length,
+                                          WMFreeDataProc *destructor);
+
+WMPropList* WMCreatePLArray(WMPropList *elem, ...);
+
+WMPropList* WMCreatePLDictionary(WMPropList *key, WMPropList *value, ...);
+
+WMPropList* WMRetainPropList(WMPropList *plist);
+
+void WMReleasePropList(WMPropList *plist);
+
+/* Objects inserted in arrays and dictionaries will be retained,
+ * so you can safely release them after insertion.
+ * For dictionaries both the key and value are retained.
+ * Objects removed from arrays or dictionaries are released */
+void WMInsertInPLArray(WMPropList *plist, int index, WMPropList *item);
+
+void WMAddToPLArray(WMPropList *plist, WMPropList *item);
+
+void WMDeleteFromPLArray(WMPropList *plist, int index);
+
+void WMRemoveFromPLArray(WMPropList *plist, WMPropList *item);
+
+void WMPutInPLDictionary(WMPropList *plist, WMPropList *key, WMPropList *value);
+
+void WMRemoveFromPLDictionary(WMPropList *plist, WMPropList *key);
+
+/* It will insert all key/value pairs from source into dest, overwriting
+ * the values with the same keys from dest, keeping all values with keys
+ * only present in dest unchanged */
+WMPropList* WMMergePLDictionaries(WMPropList *dest, WMPropList *source,
+                                  Bool recursive);
+
+/* It will remove all key/value pairs from dest for which there is an
+ * identical key/value present in source. Entires only present in dest, or
+ * which have different values in dest than in source will be preserved. */
+WMPropList* WMSubtractPLDictionaries(WMPropList *dest, WMPropList *source,
+                                     Bool recursive);
+
+int WMGetPropListItemCount(WMPropList *plist);
+
+Bool WMIsPLString(WMPropList *plist);
+
+Bool WMIsPLData(WMPropList *plist);
+
+Bool WMIsPLArray(WMPropList *plist);
+
+Bool WMIsPLDictionary(WMPropList *plist);
+
+Bool WMIsPropListEqualTo(WMPropList *plist, WMPropList *other);
+
+/* Returns a reference. Do not free it! */
+char* WMGetFromPLString(WMPropList *plist);
+
+/* Returns a reference. Do not free it! */
+WMData* WMGetFromPLData(WMPropList *plist);
+
+/* Returns a reference. Do not free it! */
+const unsigned char* WMGetPLDataBytes(WMPropList *plist);
+
+int WMGetPLDataLength(WMPropList *plist);
+
+/* Returns a reference. */
+WMPropList* WMGetFromPLArray(WMPropList *plist, int index);
+
+/* Returns a reference. */
+WMPropList* WMGetFromPLDictionary(WMPropList *plist, WMPropList *key);
+
+/* Returns a PropList array with all the dictionary keys. Release it when
+ * you're done. Keys in array are retained from the original dictionary
+ * not copied and need NOT to be released individually. */
+WMPropList* WMGetPLDictionaryKeys(WMPropList *plist);
+
+/* Creates only the first level deep object. All the elements inside are
+ * retained from the original */
+WMPropList* WMShallowCopyPropList(WMPropList *plist);
+
+/* Makes a completely separate replica of the original proplist */
+WMPropList* WMDeepCopyPropList(WMPropList *plist);
+
+WMPropList* WMCreatePropListFromDescription(char *desc);
+
+/* Free the returned string when you no longer need it */
+char* WMGetPropListDescription(WMPropList *plist, Bool indented);
+
+WMPropList* WMReadPropListFromFile(char *file);
+
+Bool WMWritePropListToFile(WMPropList *plist, char *path, Bool atomically);
+
 /*......................................................................*/
 
 WMUserDefaults* WMGetStandardUserDefaults(void);
@@ -741,14 +843,14 @@ void WMSaveUserDefaults(WMUserDefaults *database);
 
 void WMEnableUDPeriodicSynchronization(WMUserDefaults *database, Bool enable);
 
-/* Returns a PLArray with all keys in the user defaults database.
- * Free the returned array with PLRelease() when no longer needed,
- * but do not free the elements of the array! They're just references. */
-proplist_t WMGetUDAllKeys(WMUserDefaults *database);
+/* Returns a WMPropList array with all the keys in the user defaults database.
+ * Free the array with WMReleasePropList() when no longer needed.
+ * Keys in array are just retained references to the original keys */
+WMPropList* WMGetUDKeys(WMUserDefaults *database);
 
-proplist_t WMGetUDObjectForKey(WMUserDefaults *database, char *defaultName);
+WMPropList* WMGetUDObjectForKey(WMUserDefaults *database, char *defaultName);
 
-void WMSetUDObjectForKey(WMUserDefaults *database, proplist_t object,
+void WMSetUDObjectForKey(WMUserDefaults *database, WMPropList *object,
 			 char *defaultName);
 
 void WMRemoveUDObjectForKey(WMUserDefaults *database, char *defaultName);
@@ -773,9 +875,9 @@ void WMSetUDFloatForKey(WMUserDefaults *database, float value,
 void WMSetUDBoolForKey(WMUserDefaults *database, Bool value,
 		       char *defaultName);
 
-proplist_t WMGetUDSearchList(WMUserDefaults *database);
+WMPropList* WMGetUDSearchList(WMUserDefaults *database);
 
-void WMSetUDSearchList(WMUserDefaults *database, proplist_t list);
+void WMSetUDSearchList(WMUserDefaults *database, WMPropList *list);
 
 extern char *WMUserDefaultsDidChangeNotification;
 
