@@ -1,35 +1,79 @@
 
+#include "../src/config.h"
 
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
 #include <X11/Xatom.h>
+
+#ifdef SHAPE
+#include <X11/extensions/shape.h>
+#endif
+
 #include <math.h>
 
+
 #include "WINGsP.h"
 
 
 
 
-#define SPIT(a) puts(a)
+#define SPIT(a)
 
 
 #define IS_DROPPABLE(view) (view!=NULL && view->droppableTypes!=NULL && \
 				view->dragDestinationProcs!=NULL)
 
 
+static Atom operationToAction(WMScreen *scr, WMDragOperationType operation);
+static WMDragOperationType actionToOperation(WMScreen *scr, Atom action);
+
 static Bool _XErrorOccured = False;
 
 
 
+static unsigned defDraggingSourceOperation(WMView *self, Bool local)
+{
+    return WDOperationCopy;
+}
+
+
+static void defBeganDragImage(WMView *self, WMPixmap *image, WMPoint point)
+{
+}
+
+
+static void defEndedDragImage(WMView *self, WMPixmap *image, WMPoint point,
+			      Bool deposited)
+{
+}
+
+static WMData* defFetchDragData(WMView *self, char *type)
+{
+    return NULL;
+}
+
 
 void
 WMSetViewDragSourceProcs(WMView *view, WMDragSourceProcs *procs)
 {
     if (view->dragSourceProcs)
-	free(view->dragSourceProcs);
+	wfree(view->dragSourceProcs);
     view->dragSourceProcs = wmalloc(sizeof(WMDragSourceProcs));
     
     *view->dragSourceProcs = *procs;
-    
-    /* XXX fill in non-implemented stuffs */
+
+    if (procs->draggingSourceOperation == NULL) {
+	view->dragSourceProcs->draggingSourceOperation = defDraggingSourceOperation;
+    }
+    if (procs->beganDragImage == NULL) {
+	view->dragSourceProcs->beganDragImage = defBeganDragImage;
+    }
+    if (procs->endedDragImage == NULL) {
+	view->dragSourceProcs->endedDragImage = defEndedDragImage;
+    }
+    if (procs->fetchDragData == NULL) {
+	view->dragSourceProcs->fetchDragData = defFetchDragData;
+    }
 }
 
 
@@ -87,7 +131,7 @@ makeDragIcon(WMScreen *scr, WMPixmap *pixmap)
 
 #ifdef SHAPE
     if (mask) {
-	XShapeCombineMask(dpy, scr->balloon->window, ShapeBounding, 0, 0, mask,
+	XShapeCombineMask(scr->display, window, ShapeBounding, 0, 0, mask,
 			  ShapeSet);
     }
 #endif
@@ -144,15 +188,16 @@ findChildInWindow(Display *dpy, Window toplevel, int x, int y)
 	    && attr.map_state == IsViewable
 	    && x >= attr.x && y >= attr.y
 	    && x < attr.x + attr.width && y < attr.y + attr.height) {
-	    Window child;
+	    Window child, tmp;
 	    
-	    child = findChildInWindow(dpy, children[i], 
-				      x - attr.x, y - attr.y);
+	    tmp = children[i];
+	    
+	    child = findChildInWindow(dpy, tmp, x - attr.x, y - attr.y);
 
 	    XFree(children);
 	    
 	    if (child == None)
-		return toplevel;
+		return tmp;
 	    else
 		return child;
 	}
@@ -169,11 +214,12 @@ findViewInToplevel(Display *dpy, Window toplevel, int x, int y)
     Window child;
 
     child = findChildInWindow(dpy, toplevel, x, y);
-    
-    if (child != None)
+
+    if (child != None) {
 	return W_GetViewForXWindow(dpy, child);
-    else
+    } else {
 	return NULL;
+    }
 }
 
 
@@ -215,8 +261,6 @@ lookForToplevel(WMScreen *scr, Window window, Bool *isAware)
 	}
 	
 	XFree(children);
-	
-	
     }
     
     return toplevel;
@@ -317,13 +361,7 @@ sendClientMessage(Display *dpy, Window win, Atom message,
 static unsigned
 notifyPosition(WMScreen *scr, WMDraggingInfo *info)
 {
-    unsigned operation;
-	
-    switch (info->sourceOperation) {
-     default:
-	operation = None;
-	break;
-    }
+    Atom action = operationToAction(scr, info->sourceOperation);
 
     sendClientMessage(scr->display, info->destinationWindow, 
 		      scr->xdndPositionAtom,
@@ -331,7 +369,7 @@ notifyPosition(WMScreen *scr, WMDraggingInfo *info)
 		      0, /* reserved */
 		      info->location.x<<16|info->location.y,
 		      info->timestamp, 
-		      operation/* operation */);
+		      action/* operation */);
     
     return 0;
 }
@@ -388,29 +426,26 @@ translateCoordinates(WMScreen *scr, Window target, int fromX, int fromY,
 
 
 static void
-updateDraggingInfo(WMScreen *scr, WMDraggingInfo *info,
+updateDraggingInfo(WMScreen *scr, WMDraggingInfo *info, WMSize offset,
 		   XEvent *event, Window iconWindow)
 {
     Window toplevel;
-    WMSize size;
-    
-    size = WMGetPixmapSize(info->image);
 
     if (event->type == MotionNotify) {
-	info->imageLocation.x = event->xmotion.x_root-(int)size.width/2;
-	info->imageLocation.y = event->xmotion.y_root-(int)size.height/2;
+	info->imageLocation.x = event->xmotion.x_root-offset.width;
+	info->imageLocation.y = event->xmotion.y_root-offset.height;
         
 	info->location.x = event->xmotion.x_root;
 	info->location.y = event->xmotion.y_root;
-	info->timestamp = event->xmotion.time;
+/*	info->timestamp = event->xmotion.time;*/
 	
     } else if (event->type == ButtonRelease) {
-	info->imageLocation.x = event->xbutton.x_root-(int)size.width/2;
-	info->imageLocation.y = event->xbutton.y_root-(int)size.height/2;
+	info->imageLocation.x = event->xbutton.x_root-offset.width;
+	info->imageLocation.y = event->xbutton.y_root-offset.height;
         
 	info->location.x = event->xbutton.x_root;
 	info->location.y = event->xbutton.y_root;
-	info->timestamp = event->xbutton.time;
+/*	info->timestamp = event->xbutton.time;*/
     }
 
     toplevel = findToplevelUnderDragPointer(scr,
@@ -418,34 +453,6 @@ updateDraggingInfo(WMScreen *scr, WMDraggingInfo *info,
 					    info->location.y,
 					    iconWindow);
     info->destinationWindow = toplevel;
-    /*
-    if (toplevel == None) {
-	info->destinationWindow = None;
-    } else if (toplevel == scr->rootWin) {
-	info->destinationWindow = scr->rootWin;
-    } else {
-	Window child;
-	int x, y;
-	
-	XTranslateCoordinates(scr->display, scr->rootWin, toplevel,
-			      info->location.x, info->location.y,
-			      &x, &y, &child);
-
-	child = findChildInWindow(scr->display, toplevel, x, y);
-
-	if (child != None) {
-	    info->destination = W_GetViewForXWindow(scr->display, child);
-	    if (info->destination->droppableTypes == NULL) {
-		info->destination = NULL;
-	    } else if (info->destination->dragDestinationProcs == NULL) {
-		info->destination = NULL;
-	    }
-	} else {
-	    info->destination = NULL;
-	}
-	info->destinationWindow = toplevel;
-    }
-    */
 }
 
 
@@ -503,36 +510,105 @@ processMotion(WMScreen *scr, WMDraggingInfo *info, WMDraggingInfo *oldInfo,
 	    rect->size.width = 0;
 	}
     }
-    
-    /* little trick to simulate XdndStatus for local dnd */
-    /*
-    if (bla && action != currentAction) {
-	XEvent ev;
-
-	ev.type = ClientMessage;
-	ev.xclient.display = scr->display;
-	ev.xclient.message_type = scr->xdndStatusAtom;
-	ev.xclient.format = 32;
-	ev.xclient.window = info->destinationWindow;
-	ev.xclient.data.l[0] = info->sourceWindow;
-	ev.xclient.data.l[1] = (action ? 1 : 0);
-	ev.xclient.data.l[2] = 0;
-	ev.xclient.data.l[3] = 0;
-	ev.xclient.data.l[4] = action;
-	
-	XPutBackEvent(scr->display, &ev);
-    }*/
 }
 
 
 
+static WMData*
+convertSelection(WMView *view, Atom selection, Atom target,
+		 void *cdata, Atom *type)
+{
+    WMScreen *scr = W_VIEW_SCREEN(view);
+    WMData *data;
+    char *typeName = XGetAtomName(scr->display, target);
+    
+    *type = target;
+    
+    data = view->dragSourceProcs->fetchDragData(view, typeName);
+
+    if (typeName != NULL)
+	XFree(typeName);
+
+    return data;
+}
+
+
+static void 
+selectionLost(WMView *view, Atom selection, void *cdata)
+{
+    if (W_VIEW_SCREEN(view)->dragSourceView == view) {
+	wwarning("DND selection lost during drag operation...");
+	W_VIEW_SCREEN(view)->dragSourceView = NULL;
+    }
+}
+
+
+static void 
+selectionDone(WMView *view, Atom selection, Atom target, void *cdata)
+{
+    
+}
+
+
+
+
+
+static void
+setMouseOffsetHint(WMView *view, WMSize mouseOffset)
+{
+    WMScreen *scr = W_VIEW_SCREEN(view);
+    long hint[2];
+
+    /*
+     * Tell the offset from the topleft corner of the icon being
+     * dragged. Not from XDND, but it's backwards compatible.
+     */
+    
+    hint[0] = mouseOffset.width;
+    hint[1] = mouseOffset.height;
+    
+    XChangeProperty(scr->display, W_VIEW_DRAWABLE(view), 
+		    scr->wmIconDragOffsetAtom, XA_INTEGER, 32,
+		    PropModeReplace, (unsigned char*)hint, 2);
+}
+
+
+static Bool
+getMouseOffsetHint(WMScreen *scr, Window source, WMSize *mouseOffset)
+{
+    long *hint;
+    Atom type_ret;
+    int fmt_ret;
+    unsigned long nitems_ret, bytes_after_ret;
+    Bool ok = False;
+    
+
+    hint = NULL;
+    
+    XGetWindowProperty(scr->display, source, 
+		       scr->wmIconDragOffsetAtom, 0, 2, False, XA_INTEGER,
+		       &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
+		       (unsigned char **)&hint);
+    
+    if (hint && nitems_ret == 2) {
+	mouseOffset->width = hint[0];
+	mouseOffset->height = hint[1];
+	ok = True;
+    }
+    if (hint)
+	XFree(hint);
+    
+    return ok;
+}
+
+
 
 static void
 timeoutCallback(void *data)
 {
     wwarning("drag & drop timed out while waiting for response from 0x%x\n",
 	     (unsigned)data);
-    _XErrorOccured = 1;
+    _XErrorOccured = 2;
 }
 
 /*
@@ -581,9 +657,9 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 {
     WMScreen *scr = view->screen;
     Display *dpy = scr->display;
+    WMView *toplevel = W_TopLevelOfView(view);
     Window icon;
     XEvent ev;
-    WMSize size;
     WMRect rect = {{0,0},{0,0}};
     int ostate = -1;
     int state;
@@ -594,9 +670,16 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
     WMDraggingInfo dragInfo;
     WMDraggingInfo oldDragInfo;
     WMHandlerID timer = NULL;
-    WMData *draggedData = NULL;
+    static WMSelectionProcs handler = {
+	convertSelection,
+	    selectionLost,
+	    selectionDone
+    };
 
     
+    if (scr->dragSourceView != NULL)
+	return;
+    
     wassertr(view->dragSourceProcs != NULL);
 
     
@@ -604,41 +687,49 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
     if (image == NULL)
 	image = scr->defaultObjectIcon;
 
-    size = WMGetPixmapSize(image);
-
     icon = makeDragIcon(scr, image);
 
-    XMoveWindow(dpy, icon, event->xmotion.x_root-(int)size.width/2,
-		event->xmotion.y_root-(int)size.height/2);
+    XMoveWindow(dpy, icon, atLocation.x, atLocation.y);
     XMapRaised(dpy, icon);
     
 
     /* init dragging info */
+    
+    scr->dragSourceView = view;
+
     memset(&dragInfo, 0, sizeof(WMDraggingInfo));
     memset(&oldDragInfo, 0, sizeof(WMDraggingInfo));
     dragInfo.image = image;
-    dragInfo.sourceWindow = W_VIEW_DRAWABLE(W_TopLevelOfView(view));
+    dragInfo.sourceView = view;
+    dragInfo.sourceWindow = W_VIEW_DRAWABLE(toplevel);
 
     dragInfo.destinationWindow = dragInfo.sourceWindow;
 
-    dragInfo.location.x = event->xmotion.x_root;
-    dragInfo.location.y = event->xmotion.y_root;
+    dragInfo.location.x = atLocation.x + mouseOffset.width;
+    dragInfo.location.y = atLocation.y + mouseOffset.height;
     dragInfo.imageLocation = atLocation;
     
     
     /* start pointer grab */
     XGrabPointer(dpy, scr->rootWin, False,
 		 ButtonPressMask|ButtonReleaseMask|ButtonMotionMask,
-		 GrabModeSync, GrabModeAsync, None, scr->defaultCursor,
+		 GrabModeAsync, GrabModeAsync, None, scr->defaultCursor,
 		 CurrentTime);
 
     XFlush(dpy);
 
     _XErrorOccured = False;
     
-    /* XXX: take ownership of XdndSelection */
-
+    /* take ownership of XdndSelection */
+    if (!WMCreateSelectionHandler(view, scr->xdndSelectionAtom,
+				  event->xmotion.time,
+				  &handler, NULL)) {
+	wwarning("could not get ownership or DND selection");
+	return;
+    }
     
+    setMouseOffsetHint(toplevel, mouseOffset);
+
     if (view->dragSourceProcs->beganDragImage != NULL) {
 	view->dragSourceProcs->beganDragImage(view, image, atLocation);
     }
@@ -648,7 +739,6 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
     state = 1;
 
     while (state != 6 && state != 0 && !_XErrorOccured) {
-	XAllowEvents(dpy, SyncPointer, CurrentTime);
 	WMNextEvent(dpy, &ev);
 
 	switch (ev.type) {
@@ -660,14 +750,13 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 
 		oldDragInfo = dragInfo;
 
-		updateDraggingInfo(scr, &dragInfo, &ev, icon);
+		updateDraggingInfo(scr, &dragInfo, mouseOffset, &ev, icon);
 
 		XMoveWindow(dpy, icon, dragInfo.imageLocation.x,
 			    dragInfo.imageLocation.y);
 
-		if (state != 2) {
-		    processMotion(scr, &dragInfo, &oldDragInfo, &rect, action);
-		}
+		processMotion(scr, &dragInfo, &oldDragInfo, &rect, action);
+
 		protectBlock(False);
 
 		/* XXXif entered a different destination, check the operation */
@@ -675,6 +764,7 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 		switch (state) {
 		 case 1:
 		    if (oldDragInfo.destinationWindow != None
+			 && oldDragInfo.destinationWindow != scr->rootWin
 			 && (dragInfo.destinationWindow == None
 			     || dragInfo.destinationWindow == scr->rootWin)) {
 			/* left the droppable window */
@@ -684,7 +774,9 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 		    break;
 		    
 		 case 2:
-		    if (dragInfo.destinationWindow != None) {
+		    if (dragInfo.destinationWindow != None
+			&& dragInfo.destinationWindow != scr->rootWin) {
+
 			state = 1;
 			action = -1;
 		    }
@@ -693,6 +785,7 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 		 case 3:
 		 case 4:
 		    if (oldDragInfo.destinationWindow != None
+			 && oldDragInfo.destinationWindow != scr->rootWin
 			 && (dragInfo.destinationWindow == None
 			     || dragInfo.destinationWindow == scr->rootWin)) {
 			/* left the droppable window */
@@ -712,13 +805,18 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 	    
 		oldDragInfo = dragInfo;
 
-		updateDraggingInfo(scr, &dragInfo, &ev, icon);
-	    
+		updateDraggingInfo(scr, &dragInfo, mouseOffset, &ev, icon);
+
 		XMoveWindow(dpy, icon, dragInfo.imageLocation.x,
 			    dragInfo.imageLocation.y);
 
-		processMotion(scr, &dragInfo, &oldDragInfo, &rect,
-			      action);
+		if (state == 4 || state == 1) {
+		    dragInfo.destinationWindow = None;
+		    dragInfo.destView = NULL;
+		}
+		processMotion(scr, &dragInfo, &oldDragInfo, &rect, action);
+		
+		dragInfo.timestamp = ev.xbutton.time;
 
 		protectBlock(False);
 		
@@ -741,20 +839,13 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 	    }
 	    break;
 
-
-	 case SelectionRequest:
-	    
-	    draggedData = NULL;
-	    
-	    break;
-
 	 case ClientMessage:
 	    if ((state == 1 || state == 3 || state == 4 || state == 5)
 		&& ev.xclient.message_type == scr->xdndStatusAtom
-		&& ev.xclient.window == dragInfo.destinationWindow) {
+		&& ev.xclient.data.l[0] == dragInfo.destinationWindow) {
 
 		if (ev.xclient.data.l[1] & 1) {
-		    puts("got accept msg");
+		    SPIT("got accept msg");
 		    /* will accept drop */
 		    switch (state) {
 		     case 1:
@@ -770,13 +861,9 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 			state = 6;
 			break;
 		    }
-		    if (ev.xclient.data.l[4] == None) {
-			action = 0;
-		    } else {
-			action = ev.xclient.data.l[4];/*XXX*/
-		    }
+		    action = actionToOperation(scr, ev.xclient.data.l[4]);
 		} else {
-		    puts("got reject msg");
+		    SPIT("got reject msg");
 		    switch (state) {
 		     case 1:
 		     case 3:
@@ -825,7 +912,6 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 	}
 	
 	if (ostate != state) {
-	    printf("state changed to %i\n", state);
 	    if (state == 3) {
 		XRecolorCursor(dpy, scr->defaultCursor, &green, &back);
 	    } else if (ostate == 3) {
@@ -834,10 +920,13 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
 	    ostate = state;
 	}
     }
-
+    
     if (timer) {
 	WMDeleteTimerHandler(timer);
 	timer = NULL;
+    } else if (_XErrorOccured) {
+	/* got a timeout, send leave */
+	notifyDragLeave(scr, &dragInfo);
     }
 
     XUngrabPointer(dpy, CurrentTime);
@@ -859,23 +948,18 @@ WMDragImageFromView(WMView *view, WMPixmap *image, char *dataTypes[],
     
     
     SPIT("dropped");
-    /* wait for Finished message and SelectionRequest if not a local drop */
-    
     
 
     XDestroyWindow(dpy, icon);
-    if (view->dragSourceProcs->endedDragImage != NULL) {
-	view->dragSourceProcs->endedDragImage(view, image, 
-					      dragInfo.imageLocation,
-					      True);
-    }
+    
     return;
 
 cancelled:
-    if (draggedData) {
-	WMReleaseData(draggedData);
-    }
+    scr->dragSourceView = NULL;
     
+    WMDeleteSelectionHandler(view, scr->xdndSelectionAtom, 
+			     event->xmotion.time);
+
     if (slideBack) {
 	slideWindow(dpy, icon,
 		    dragInfo.imageLocation.x, dragInfo.imageLocation.y,
@@ -900,8 +984,67 @@ cancelled:
 
 
 
+static Atom
+operationToAction(WMScreen *scr, WMDragOperationType operation)
+{
+    switch (operation) {
+     case WDOperationNone:
+	return None;
+	    
+     case WDOperationCopy:
+	return scr->xdndActionCopy;
+	    
+     case WDOperationMove:
+	return scr->xdndActionMove;
+	    
+     case WDOperationLink:
+	return scr->xdndActionLink;
+	
+     case WDOperationAsk:
+	return scr->xdndActionAsk;
+	    
+     case WDOperationPrivate:
+	return scr->xdndActionPrivate;
+	    
+     default:
+	return None;
+    }
+}
 
 
+static WMDragOperationType
+actionToOperation(WMScreen *scr, Atom action)
+{
+    if (action == scr->xdndActionCopy) {
+	return WDOperationCopy;
+	
+    } else if (action == scr->xdndActionMove) {
+	return WDOperationMove;
+	
+    } else if (action == scr->xdndActionLink) {
+	return WDOperationLink;
+	
+    } else if (action == scr->xdndActionAsk) {
+	return WDOperationAsk;
+	
+    } else if (action == scr->xdndActionPrivate) {
+	return WDOperationPrivate;
+	
+    } else if (action == None) {
+	
+	return WDOperationCopy;
+    } else {
+	char *tmp = XGetAtomName(scr->display, action);
+	
+	wwarning("unknown XDND action %s from 0x%x", tmp,
+		 (unsigned)scr->dragInfo.sourceWindow);
+	XFree(tmp);
+	
+	return WDOperationCopy;
+    }
+}
+    
+
 
 
 
@@ -934,21 +1077,85 @@ getTypeList(Window window, XClientMessageEvent *event)
 }
 
 
+
+#define DISPATCH(view, func, info) (view)->dragDestinationProcs->func(view, info)
+
+
+
+static void
+receivedData(WMView *view, Atom selection, Atom target,
+	     Time timestamp, void *cdata, WMData *data)
+{
+    WMScreen *scr = W_VIEW_SCREEN(view);
+    WMDraggingInfo *info = (WMDraggingInfo*)cdata;
+    Bool res;
+
+    res = view->dragDestinationProcs->performDragOperation(view, info);
+
+    if (res) {
+	DISPATCH(view, concludeDragOperation, info);
+    }
+
+    /* send finished message */
+    sendClientMessage(scr->display, info->sourceWindow,
+		      scr->xdndFinishedAtom,
+		      info->destinationWindow,
+		      0, 0, 0, 0);
+
+    memset(&scr->dragInfo, 0, sizeof(WMDraggingInfo));
+}
+
+
+
 void
 W_HandleDNDClientMessage(WMView *toplevel, XClientMessageEvent *event)
 {
-#if 0
     WMScreen *scr = W_VIEW_SCREEN(toplevel);
-    WMView *oldView = NULL, *newView = NULL;
+    WMView *oldView = NULL;
+    WMView *newView = NULL;
     unsigned operation = 0;
     int x, y;
+    enum {
+	WNothing,
+	WEnter,
+	WLeave,
+	WCross, /* leave one and enter another */
+	WUpdate,
+	WDrop
+    };
+    Window source;
+    int what = WNothing;
+    Bool sendStatus = False;
+    
+
+    source = scr->dragInfo.sourceWindow;
+    oldView = scr->dragInfo.destView;
+
+    if (event->message_type == scr->xdndFinishedAtom) {
+	WMView *view = scr->dragSourceView;
+	
+	WMDeleteSelectionHandler(view, scr->xdndSelectionAtom,
+				 scr->dragInfo.timestamp);
+	
+	if (view->dragSourceProcs->endedDragImage != NULL) {
+	    view->dragSourceProcs->endedDragImage(view,
+						  scr->dragInfo.image,
+						  scr->dragInfo.imageLocation,
+						  True);
+	}
+	
+	scr->dragSourceView = NULL;
+	
+	return;
+    }
+    
+    
 
     if (event->message_type == scr->xdndEnterAtom) {
 	Window foo, bar;
 	int bla;
 	unsigned ble;
-	puts("entered");
-	
+
 	if (scr->dragInfo.sourceWindow != None) {
 	    puts("received Enter event in bad order");
 	}
@@ -956,22 +1163,28 @@ W_HandleDNDClientMessage(WMView *toplevel, XClientMessageEvent *event)
 	memset(&scr->dragInfo, 0, sizeof(WMDraggingInfo));
 	
 	
-	if ((event->data.l[0] >> 24) > XDND_VERSION) {
+	if ((event->data.l[1] >> 24) > XDND_VERSION) {
 	    wwarning("received drag & drop request with unsupported version %i",
-		    (event->data.l[0] >> 24));
+		    (event->data.l[1] >> 24));
 	    return;
 	}
 	
 	scr->dragInfo.protocolVersion = event->data.l[1] >> 24;
-	scr->dragInfo.sourceWindow = event->data.l[0];
+	scr->dragInfo.sourceWindow = source = event->data.l[0];
 	scr->dragInfo.destinationWindow = event->window;
 	
+	getMouseOffsetHint(scr, source, &scr->dragInfo.mouseOffset);
+	
 	/* XXX */
 	scr->dragInfo.image = NULL;
 	
 	XQueryPointer(scr->display, scr->rootWin, &foo, &bar,
 		      &scr->dragInfo.location.x, &scr->dragInfo.location.y,
 		      &bla, &bla, &ble);
+	
+	scr->dragInfo.imageLocation = scr->dragInfo.location;
+	scr->dragInfo.imageLocation.x -= scr->dragInfo.mouseOffset.width;
+	scr->dragInfo.imageLocation.y -= scr->dragInfo.mouseOffset.height;
 
 	translateCoordinates(scr, scr->dragInfo.destinationWindow,
 			     scr->dragInfo.location.x,
@@ -979,105 +1192,140 @@ W_HandleDNDClientMessage(WMView *toplevel, XClientMessageEvent *event)
 
 
 	newView = findViewInToplevel(scr->display,
-		 		  scr->dragInfo.destinationWindow, 
-				  x, y);
-	
-	if (IS_DROPPABLE(view)) {
-	    
-	    scr->dragInfo.destinationView = view;
-	    
-	    operation =
-		view->dragDestinationProcs->draggingEntered(view, 
-							    &scr->dragInfo);
-	}
-	if (operation > 0) {
-	    Atom action;
-		
-	    switch (operation) {
-	     default:
-		action = 0;
-		break;
-	    }
-
-	    sendClientMessage(scr->display, scr->dragInfo.sourceWindow,
-			      scr->xdndStatusAtom,
-			      scr->dragInfo.destinationWindow,
-			      1, 0, 0, action);
-	}
+				     scr->dragInfo.destinationWindow, 
+				     x, y);
 
     } else if (event->message_type == scr->xdndPositionAtom
 	       && scr->dragInfo.sourceWindow == event->data.l[0]) {
-	
+
 	scr->dragInfo.location.x = event->data.l[2] >> 16;
 	scr->dragInfo.location.y = event->data.l[2] & 0xffff;
-	
+
+	scr->dragInfo.imageLocation = scr->dragInfo.location;
+	scr->dragInfo.imageLocation.x -= scr->dragInfo.mouseOffset.width;
+	scr->dragInfo.imageLocation.y -= scr->dragInfo.mouseOffset.height;
+
 	if (scr->dragInfo.protocolVersion >= 1) {
 	    scr->dragInfo.timestamp = event->data.l[3];
-	    scr->dragInfo.sourceOperation = event->data.l[4];
+	    scr->dragInfo.sourceOperation = actionToOperation(scr,
+							    event->data.l[4]);
+
 	} else {
 	    scr->dragInfo.timestamp = CurrentTime;
-	    scr->dragInfo.sourceOperation = 0; /*XXX*/
+	    scr->dragInfo.sourceOperation = WDOperationCopy;
 	}
 
 	translateCoordinates(scr, scr->dragInfo.destinationWindow,
 			     scr->dragInfo.location.x,
 			     scr->dragInfo.location.y, &x, &y);
 
-	view = findViewInToplevel(scr->display, 
-				  scr->dragInfo.destinationWindow,
-				  x, y);
-	
-	if (scr->dragInfo.destinationView != view) {
-	    WMView *oldVIew = scr->dragInfo.destinationView;
-	    
-	    oldView->dragDestinationProcs->draggingExited(oldView,
-							  &scr->dragInfo);
-	    
-	    scr->dragInfo.destinationView = NULL;
-	}
-	
-
-	if (IS_DROPPABLE(view)) {
-	    
-	    
-	    operation =
-		view->dragDestinationProcs->draggingUpdated(view,
-							    &scr->dragInfo);
-	}
-	
-	if (operation == 0) {
-	    sendClientMessage(scr->display, scr->dragInfo.sourceWindow,
-			      scr->xdndStatusAtom,
-			      scr->dragInfo.destinationWindow,
-			      0, 0, 0, None);
-	} else {
-	    Atom action;
-		
-	    switch (operation) {
-	     default:
-		action = 0;
-		break;
-	    }
-
-	    sendClientMessage(scr->display, scr->dragInfo.sourceWindow,
-			      scr->xdndStatusAtom,
-			      scr->dragInfo.destinationWindow,
-			      1, 0, 0, action);
-	}
+	newView = findViewInToplevel(scr->display, 
+				     scr->dragInfo.destinationWindow,
+				     x, y);
 	
     } else if (event->message_type == scr->xdndLeaveAtom
 	       && scr->dragInfo.sourceWindow == event->data.l[0]) {
 
-	void (*draggingExited)(WMView *self, WMDraggingInfo *info);
-	
-	puts("leave");
+	memset(&scr->dragInfo, 0, sizeof(WMDraggingInfo));
+
     } else if (event->message_type == scr->xdndDropAtom
 	       && scr->dragInfo.sourceWindow == event->data.l[0]) {
+
+	/* drop */
+	if (oldView != NULL)
+	    what = WDrop;
+
+    } else {
+	return;
+    }
+
+
+    /*
+     * Now map the XDND events to WINGs events.
+     */
+
+    if (what == WNothing) {
+	if (IS_DROPPABLE(newView)) {
+	    if (!IS_DROPPABLE(oldView)) { /* entered */
+		what = WEnter;
+	    } else if (oldView == newView) { /* updated */
+		what = WUpdate;
+	    } else {
+		what = WCross;
+	    }
+	} else {
+	    if (IS_DROPPABLE(oldView)) {
+		what = WLeave;
+	    } else {
+		/* just send rejection msg */
+		sendStatus = True;
+	    }
+	}
+    }
+    
+    
+    
+    switch (what) {
+
+     case WEnter:
+	scr->dragInfo.destView = newView;
+	operation = DISPATCH(newView, draggingEntered, &scr->dragInfo);
+	sendStatus = True;
+	break;
 	
+     case WLeave:
+	scr->dragInfo.destView = NULL;
+	DISPATCH(oldView, draggingExited, &scr->dragInfo);
+	sendStatus = True;
+	operation = WDOperationNone;
+	break;
 	
+     case WCross:
+	DISPATCH(oldView, draggingExited, &scr->dragInfo);
+	scr->dragInfo.destView = newView;
+	operation = DISPATCH(newView, draggingEntered, &scr->dragInfo);
+	sendStatus = True;
+	break;
 	
-	puts("drop");
+     case WUpdate:
+	operation = DISPATCH(oldView, draggingUpdated, &scr->dragInfo);
+	sendStatus = True;
+	break;
+	
+     case WDrop:
+	{
+	    Bool res;
+	    
+	    res = DISPATCH(oldView, prepareForDragOperation, &scr->dragInfo);
+
+	    if (res) {
+		res = DISPATCH(oldView, performDragOperation, &scr->dragInfo);
+	    }
+	    
+	    if (res) {
+		
+	    }
+	}
+	break;
+	
+     default:
+	break;
+    }
+    
+    
+    
+    if (sendStatus) {
+	Atom action;
+
+	action = operationToAction(scr, operation);
+
+	sendClientMessage(scr->display, source,
+			  scr->xdndStatusAtom,
+			  scr->dragInfo.destinationWindow,
+			  action != None ? 1 : 0, 0, 0, action);
     }
-#endif
 }
 
+
+
+
