@@ -409,7 +409,7 @@ indentedDescription(WMPropList *plist, int level)
     WMHashEnumerator e;
     int i;
 
-    if (plist->type==WPLArray || plist->type==WPLDictionary) {
+    if (plist->type==WPLArray/* || plist->type==WPLDictionary*/) {
         retstr = description(plist);
 
         if (retstr && ((2*(level+1) + strlen(retstr)) <= 77)) {
@@ -730,7 +730,7 @@ getPLArray(PLData *pldata)
         } else if (c == ',') {
             /* continue normally */
         } else if (!first) {
-            COMPLAIN(pldata, _("missing , or unterminated PropList array"));
+            COMPLAIN(pldata, _("missing or unterminated PropList array"));
             ok = 0;
             break;
         } else {
@@ -1211,16 +1211,63 @@ WMRemoveFromPLDictionary(WMPropList *plist, WMPropList *key)
 
 
 WMPropList*
-WMMergePLDictionaries(WMPropList *dest, WMPropList *source)
+WMMergePLDictionaries(WMPropList *dest, WMPropList *source, Bool recursive)
 {
-    WMPropList *key, *value;
+    WMPropList *key, *value, *dvalue;
     WMHashEnumerator e;
 
     wassertr(source->type==WPLDictionary && dest->type==WPLDictionary);
 
+    if (source == dest)
+        return dest;
+
     e = WMEnumerateHashTable(source->d.dict);
     while (WMNextHashEnumeratorItemAndKey(&e, (void**)&value, (void**)&key)) {
-        WMPutInPLDictionary(dest, key, value);
+        if (recursive && value->type==WPLDictionary) {
+            dvalue = WMHashGet(dest->d.dict, key);
+            if (dvalue && dvalue->type==WPLDictionary) {
+                WMMergePLDictionaries(dvalue, value, True);
+            } else {
+                WMPutInPLDictionary(dest, key, value);
+            }
+        } else {
+            WMPutInPLDictionary(dest, key, value);
+        }
+    }
+
+    return dest;
+}
+
+
+WMPropList*
+WMSubtractPLDictionaries(WMPropList *dest, WMPropList *source, Bool recursive)
+{
+    WMPropList *key, *value, *dvalue;
+    WMHashEnumerator e;
+
+    wassertr(source->type==WPLDictionary && dest->type==WPLDictionary);
+
+    if (source == dest) {
+        WMPropList *keys = WMGetPLDictionaryKeys(dest);
+        int i;
+
+        for (i=0; i<WMGetArrayItemCount(keys->d.array); i++) {
+            WMRemoveFromPLDictionary(dest, WMGetFromArray(keys->d.array, i));
+        }
+        return dest;
+    }
+
+    e = WMEnumerateHashTable(source->d.dict);
+    while (WMNextHashEnumeratorItemAndKey(&e, (void**)&value, (void**)&key)) {
+        dvalue = WMHashGet(dest->d.dict, key);
+        if (!dvalue)
+            continue;
+        if (WMIsPropListEqualTo(value, dvalue)) {
+            WMRemoveFromPLDictionary(dest, key);
+        } else if (recursive && value->type==WPLDictionary &&
+            dvalue->type==WPLDictionary) {
+            WMSubtractPLDictionaries(dvalue, value, True);
+        }
     }
 
     return dest;
@@ -1544,14 +1591,15 @@ WMReadPropListFromFile(char *file)
 
     f = fopen(file, "r");
     if (!f) {
-        wsyserror(_("could not open domain file %s for reading"), file);
+        /* let the user print the error message if he really needs to */
+        /*wsyserror(_("could not open domain file '%s' for reading"), file);*/
         return NULL;
     }
 
     if (stat(file, &stbuf)==0) {
         length = (size_t) stbuf.st_size;
     } else {
-        wsyserror(_("could not get size for domain file %s"), file);
+        wsyserror(_("could not get size for file '%s'"), file);
         fclose(f);
         return NULL;
     }
@@ -1563,7 +1611,7 @@ WMReadPropListFromFile(char *file)
     pldata->lineNumber = 1;
 
     if (fread(pldata->ptr, length, 1, f) != 1) {
-        wsyserror(_("error reading from file %s"), file);
+        wsyserror(_("error reading from file '%s'"), file);
         plist = NULL;
         goto cleanup;
     }
