@@ -69,6 +69,7 @@ WMCreateLabel(WMWidget *parent)
     lPtr->flags.alignment = DEFAULT_ALIGNMENT;
     lPtr->flags.relief = DEFAULT_RELIEF;
     lPtr->flags.imagePosition = DEFAULT_IMAGE_POSITION;
+    lPtr->flags.noWrap = 1;
 
     return lPtr;
 }
@@ -98,6 +99,13 @@ WMGetLabelImage(WMLabel *lPtr)
 }
 
 
+char *
+WMGetLabelText(WMLabel *lPtr)
+{
+    return lPtr->caption;
+}
+
+
 void
 WMSetLabelImagePosition(WMLabel *lPtr, WMImagePosition position)
 {
@@ -145,6 +153,13 @@ WMSetLabelText(WMLabel *lPtr, char *text)
 }
 
 
+WMFont*
+WMGetLabelFont(WMLabel *lPtr)
+{
+    return lPtr->font;
+}
+
+
 void
 WMSetLabelFont(WMLabel *lPtr, WMFont *font)
 {	
@@ -167,6 +182,10 @@ WMSetLabelTextColor(WMLabel *lPtr, WMColor *color)
     if (lPtr->textColor)
 	WMReleaseColor(lPtr->textColor);
     lPtr->textColor = WMRetainColor(color);
+
+    if (lPtr->view->flags.realized) {
+	paintLabel(lPtr);
+    }
 }
 
 
