@@ -4,12 +4,6 @@
 
 #include "WINGsP.h"
 
-typedef struct ItemList {
-    char *text;
-    struct ItemList *nextPtr;
-    unsigned int disabled:1;
-} ItemList;
-
 
 typedef struct W_PopUpButton {
     W_Class widgetClass;
@@ -20,14 +14,11 @@ typedef struct W_PopUpButton {
 
     char *caption;
 
-    ItemList *items;
-    short itemCount;
-
+    WMArray *items;
+    
     short selectedItemIndex;
     
     short highlightedItem;
-
-    ItemList *selectedItem;	       /* selected item if it is a menu btn */
     
     WMView *menuView;		       /* override redirect popup menu */
 
@@ -52,14 +43,7 @@ typedef struct W_PopUpButton {
 #define MENU_BLINK_DELAY	60000
 #define MENU_BLINK_COUNT	2
 
-#define SCROLL_DELAY		30
-
-
-W_ViewProcedureTable _PopUpButtonViewProcedures = {
-    NULL,
-	NULL,
-	NULL
-};
+#define SCROLL_DELAY		10
 
 
 #define DEFAULT_WIDTH	60
@@ -90,7 +74,7 @@ WMCreatePopUpButton(WMWidget *parent)
     
     bPtr->view = W_CreateView(W_VIEW(parent));
     if (!bPtr->view) {
-	free(bPtr);
+	wfree(bPtr);
 	return NULL;
     }
     bPtr->view->self = bPtr;
@@ -107,9 +91,12 @@ WMCreatePopUpButton(WMWidget *parent)
 
     bPtr->flags.enabled = 1;
     
-    bPtr->menuView = W_CreateTopView(scr);
-    bPtr->menuView->attribs.override_redirect = True;
-    bPtr->menuView->attribFlags |= CWOverrideRedirect;
+    bPtr->items =
+        WMCreateArrayWithDestructor(4, (WMFreeDataProc*)WMDestroyMenuItem);
+
+    bPtr->selectedItemIndex = -1;
+
+    bPtr->menuView = W_CreateUnmanagedTopView(scr);
 
     W_ResizeView(bPtr->menuView, bPtr->view->size.width, 1);
 
@@ -132,69 +119,36 @@ WMSetPopUpButtonAction(WMPopUpButton *bPtr, WMAction *action, void *clientData)
 }
 
 
-void
+WMMenuItem*
 WMAddPopUpButtonItem(WMPopUpButton *bPtr, char *title)
 {
-    ItemList *itemPtr, *tmp;
+    WMMenuItem *item;
     
     CHECK_CLASS(bPtr, WC_PopUpButton);
 
-    itemPtr = wmalloc(sizeof(ItemList));
-    memset(itemPtr, 0, sizeof(ItemList));
-    itemPtr->text = wstrdup(title);
-    
-    /* append item to list */
-    tmp = bPtr->items;
-    if (!tmp)
-	bPtr->items = itemPtr;
-    else {
-	while (tmp->nextPtr!=NULL) 
-	    tmp = tmp->nextPtr;
-	tmp->nextPtr = itemPtr;
-    }
-    
-    bPtr->itemCount++;
-    
+    item = WMCreateMenuItem();
+    WMSetMenuItemTitle(item, title);
+
+    WMAddToArray(bPtr->items, item);
+
     if (bPtr->menuView && bPtr->menuView->flags.realized)
 	resizeMenu(bPtr);
+
+    return item;
 }
 
 
-void
+WMMenuItem*
 WMInsertPopUpButtonItem(WMPopUpButton *bPtr, int index, char *title)
 {
-    ItemList *itemPtr;
+    WMMenuItem *item;
 
     CHECK_CLASS(bPtr, WC_PopUpButton);
     
-    if (index < 0)
-	index = 0;
-    if (index >= bPtr->itemCount) {
-	WMAddPopUpButtonItem(bPtr, title);
-	return;
-    }
-    
-    itemPtr = wmalloc(sizeof(ItemList));
-    memset(itemPtr, 0, sizeof(ItemList));
-    itemPtr->text = wstrdup(title);
+    item = WMCreateMenuItem();
+    WMSetMenuItemTitle(item, title);
 
-    if (index == 0) {
-	itemPtr->nextPtr = bPtr->items;
-	bPtr->items = itemPtr;
-    } else {
-	ItemList *tmp;
-	int i = index;
-	
-	tmp = bPtr->items;
-	/* insert item in list */
-	while (--i > 0) {
-	    tmp = tmp->nextPtr;
-	}
-	itemPtr->nextPtr = tmp->nextPtr;
-	tmp->nextPtr = itemPtr;
-    }
-    
-    bPtr->itemCount++;
+    WMInsertInArray(bPtr->items, index, item);
   
     /* if there is an selected item, update it's index to match the new 
      * position */
@@ -203,49 +157,27 @@ WMInsertPopUpButtonItem(WMPopUpButton *bPtr, int index, char *title)
     
     if (bPtr->menuView && bPtr->menuView->flags.realized)
 	resizeMenu(bPtr);
+    
+    return item;
 }
 
 
 void
 WMRemovePopUpButtonItem(WMPopUpButton *bPtr, int index)
 {
-    ItemList *tmp;
-
     CHECK_CLASS(bPtr, WC_PopUpButton);
-    
-    if (index < 0 || index >= bPtr->itemCount)
-	return;
+
+    wassertr(index >= 0 && index < WMGetArrayItemCount(bPtr->items));
 
 
-    if (index == 0) {
-	free(bPtr->items->text);
-	tmp = bPtr->items->nextPtr;
-	free(bPtr->items);
-	bPtr->items = tmp;
-    } else {
-	ItemList *next;
-	int i = index;
-	
-	tmp = bPtr->items;
-	while (--i > 0)
-	    tmp = tmp->nextPtr;
-	next = tmp->nextPtr->nextPtr;
-	
-	free(tmp->nextPtr->text);
-	free(tmp->nextPtr);
+    WMDeleteFromArray(bPtr->items, index);
 
-	tmp->nextPtr = next;
-    }
-
-    bPtr->itemCount--;
-
-    if (bPtr->selectedItem!=NULL && !bPtr->flags.pullsDown) {
+    if (bPtr->selectedItemIndex >= 0 && !bPtr->flags.pullsDown) {
 	if (index < bPtr->selectedItemIndex)
 	    bPtr->selectedItemIndex--;
 	else if (index == bPtr->selectedItemIndex) {
-	    /* reselect first item if the removed item is the 
+	    /* reselect first item if the removed item is the
 	     * selected one */
-	    bPtr->selectedItem = bPtr->items;
 	    bPtr->selectedItemIndex = 0;
 	    if (bPtr->view->flags.mapped)
 		paintPopUpButton(bPtr);
@@ -266,33 +198,33 @@ WMSetPopUpButtonEnabled(WMPopUpButton *bPtr, Bool flag)
 }
 
 
+Bool
+WMGetPopUpButtonEnabled(WMPopUpButton *bPtr)
+{
+    return bPtr->flags.enabled;
+}
+
+
 void
 WMSetPopUpButtonSelectedItem(WMPopUpButton *bPtr, int index)
 {
-    ItemList *itemPtr = bPtr->items;
-    int i = index;
-    
-    if (index < 0) {
-	bPtr->selectedItem = NULL;
-	if (bPtr->view->flags.mapped)
-	    paintPopUpButton(bPtr);
-	return;
-    }
-    
-    while (i-- > 0) {
-	itemPtr = itemPtr->nextPtr;
-    }
-    bPtr->selectedItem = itemPtr;
+
+    wassertr(index < WMGetArrayItemCount(bPtr->items));
+
+    /* if (index >= WMGetArrayCount(bPtr->items))
+        index = -1;*/
+
     bPtr->selectedItemIndex = index;
     
     if (bPtr->view->flags.mapped)
 	paintPopUpButton(bPtr);
 }
 
+
 int
 WMGetPopUpButtonSelectedItem(WMPopUpButton *bPtr)
 {
-    if (!bPtr->flags.pullsDown && bPtr->selectedItem==NULL)
+    if (!bPtr->flags.pullsDown && bPtr->selectedItemIndex < 0)
 	return -1;
     else
 	return bPtr->selectedItemIndex;
@@ -303,7 +235,7 @@ void
 WMSetPopUpButtonText(WMPopUpButton *bPtr, char *text)
 {
     if (bPtr->caption)
-	free(bPtr->caption);
+	wfree(bPtr->caption);
     if (text)
 	bPtr->caption = wstrdup(text);
     else
@@ -320,16 +252,14 @@ WMSetPopUpButtonText(WMPopUpButton *bPtr, char *text)
 void 
 WMSetPopUpButtonItemEnabled(WMPopUpButton *bPtr, int index, Bool flag)
 {
-    int i;
-    ItemList *item = bPtr->items;
-    
-    if (index < 0 || index >= bPtr->itemCount)
-	return;
-    
-    for (i = 0; i<index; i++)
-	item=item->nextPtr;
-    
-    item->disabled = !flag;
+    WMSetMenuItemEnabled(WMGetFromArray(bPtr->items, index), flag);
+}
+
+
+Bool
+WMGetPopUpButtonItemEnabled(WMPopUpButton *bPtr, int index)
+{
+    return WMGetMenuItemEnabled(WMGetFromArray(bPtr->items, index));
 }
 
 
@@ -337,22 +267,9 @@ void
 WMSetPopUpButtonPullsDown(WMPopUpButton *bPtr, Bool flag)
 {
     bPtr->flags.pullsDown = flag;
-    if (!flag) {
-        /* This code causes bugs. It should not select any item,
-         * since it was not asked to. -Dan
-        bPtr->selectedItem = bPtr->items;
-	if (bPtr->selectedItem)
-	    bPtr->selectedItemIndex = 0;
-	else
-            bPtr->selectedItemIndex = -1;
-        */
-        bPtr->selectedItem = NULL;
-        /* the drawing routine, still draws things wrong if we put
-         * the index to -1 (i.e. not selected).
-         * Find out why. -Dan */
-        bPtr->selectedItemIndex = 0;
-    } else
+    if (flag) {
 	bPtr->selectedItemIndex = -1;
+    }
 
     if (bPtr->view->flags.mapped)
 	paintPopUpButton(bPtr);
@@ -362,25 +279,28 @@ WMSetPopUpButtonPullsDown(WMPopUpButton *bPtr, Bool flag)
 int
 WMGetPopUpButtonNumberOfItems(WMPopUpButton *bPtr)
 {
-    return bPtr->itemCount;
+    return WMGetArrayItemCount(bPtr->items);
 }
 
 
 char*
 WMGetPopUpButtonItem(WMPopUpButton *bPtr, int index)
 {
-    ItemList *itemPtr = bPtr->items;
-    
-    if ((index < 0) || (index >= bPtr->itemCount))
+    if (index >= WMGetArrayItemCount(bPtr->items) || index < 0)
 	return NULL;
-    
-    while (index-->0)
-	itemPtr = itemPtr->nextPtr;
-    
-    return itemPtr->text;
+
+    return WMGetMenuItemTitle(WMGetFromArray(bPtr->items, index));
 }
 
 
+WMMenuItem*
+WMGetPopUpButtonMenuItem(WMPopUpButton *bPtr, int index)
+{
+    return WMGetFromArray(bPtr->items, index);
+}
+
+
+
 static void
 paintPopUpButton(PopUpButton *bPtr)
 {
@@ -392,11 +312,11 @@ paintPopUpButton(PopUpButton *bPtr)
     if (bPtr->flags.pullsDown) {
 	caption = bPtr->caption;
     } else {
-	if (bPtr->selectedItem == NULL) {
+	if (bPtr->selectedItemIndex < 0) {
 	    /* if no item selected, show the caption */
 	    caption = bPtr->caption;
 	} else {
-	    caption = bPtr->selectedItem->text;
+	    caption = WMGetPopUpButtonItem(bPtr, bPtr->selectedItemIndex);
 	}
     }
 
@@ -471,14 +391,17 @@ static void
 paintMenuEntry(PopUpButton *bPtr, int index, int highlight)
 {
     W_Screen *scr = bPtr->view->screen;
-    int i;
     int yo;
-    ItemList *itemPtr;
-    int width, height, itemHeight;
-    
+    int width, height, itemHeight, itemCount;
+    char *title;
+
+    itemCount = WMGetArrayItemCount(bPtr->items);
+    if (index < 0 || index >= itemCount)
+        return;
+
     itemHeight = bPtr->view->size.height;
     width = bPtr->view->size.width;
-    height = itemHeight * bPtr->itemCount;
+    height = itemHeight * itemCount;
     yo = (itemHeight - WMFontHeight(scr->normalFont))/2;
 
     if (!highlight) {
@@ -492,23 +415,21 @@ paintMenuEntry(PopUpButton *bPtr, int index, int highlight)
     XFillRectangle(scr->display, bPtr->menuView->window, WMColorGC(scr->white),
 		   1, index*itemHeight+1, width-3, itemHeight-3);
     
-    itemPtr = bPtr->items;
-    for (i = 0; i < index; i++)
-	itemPtr = itemPtr->nextPtr;
-    
+    title = WMGetPopUpButtonItem(bPtr, index);
+
     W_DrawRelief(scr, bPtr->menuView->window, 0, index*itemHeight, 
 		 width, itemHeight, WRRaised);
 
     W_PaintText(bPtr->menuView, bPtr->menuView->window, scr->normalFont,  6, 
-		index*itemHeight + yo, width, WALeft, WMColorGC(scr->black), False,
-		itemPtr->text, strlen(itemPtr->text));
-	
+		index*itemHeight + yo, width, WALeft, WMColorGC(scr->black), 
+		False, title, strlen(title));
+
     if (!bPtr->flags.pullsDown && index == bPtr->selectedItemIndex) {
 	XCopyArea(scr->display, scr->popUpIndicator->pixmap, 
 		  bPtr->menuView->window, scr->copyGC, 0, 0, 
 		  scr->popUpIndicator->width, scr->popUpIndicator->height,
 		  width-scr->popUpIndicator->width-4, 
-		  i*itemHeight+(itemHeight-scr->popUpIndicator->height)/2);
+		  index*itemHeight+(itemHeight-scr->popUpIndicator->height)/2);
     }
 }
 
@@ -518,36 +439,40 @@ makeMenuPixmap(PopUpButton *bPtr)
 {
     Pixmap pixmap;
     W_Screen *scr = bPtr->view->screen;
-    int i;
-    int yo;
-    ItemList *itemPtr;
+    WMMenuItem *item;
+    WMArrayIterator iter;
+    int yo, i;
     int width, height, itemHeight;
     
     itemHeight = bPtr->view->size.height;
     width = bPtr->view->size.width;
-    height = itemHeight * bPtr->itemCount;
+    height = itemHeight * WMGetArrayItemCount(bPtr->items);
     yo = (itemHeight - WMFontHeight(scr->normalFont))/2;
     
     pixmap = XCreatePixmap(scr->display, bPtr->view->window, width, height, 
 			   scr->depth);
     
-    XFillRectangle(scr->display, pixmap, WMColorGC(scr->gray), 0, 0, width, height);
-    
-    itemPtr = bPtr->items;
-    for (i = 0; i < bPtr->itemCount; i++) {
+    XFillRectangle(scr->display, pixmap, WMColorGC(scr->gray), 0, 0,
+		   width, height);
+
+    i = 0;
+    WM_ITERATE_ARRAY(bPtr->items, item, iter) {
 	GC gc;
-	
+	char *text;
+
+	text = WMGetMenuItemTitle(item);
+
 	W_DrawRelief(scr, pixmap, 0, i*itemHeight, width, itemHeight, 
 		     WRRaised);
 
-        if (itemPtr->disabled)
+        if (!WMGetMenuItemEnabled(item))
 	    gc = WMColorGC(scr->darkGray);
 	else
 	    gc = WMColorGC(scr->black);
 
 	W_PaintText(bPtr->menuView, pixmap, scr->normalFont,  6, 
 		    i*itemHeight + yo, width, WALeft, gc, False,
-		    itemPtr->text, strlen(itemPtr->text));
+		    text, strlen(text));
 	
 	if (!bPtr->flags.pullsDown && i == bPtr->selectedItemIndex) {
 	    XCopyArea(scr->display, scr->popUpIndicator->pixmap, pixmap, 
@@ -555,8 +480,9 @@ makeMenuPixmap(PopUpButton *bPtr)
 		      scr->popUpIndicator->height, 
 		      width-scr->popUpIndicator->width-4,
 		      i*itemHeight+(itemHeight-scr->popUpIndicator->height)/2);
-	}	
-	itemPtr = itemPtr->nextPtr;
+	}
+	
+	i++;
     }
     
     return pixmap;
@@ -568,7 +494,7 @@ resizeMenu(PopUpButton *bPtr)
 {
     int height;
     
-    height = bPtr->itemCount * bPtr->view->size.height;
+    height = WMGetArrayItemCount(bPtr->items) * bPtr->view->size.height;
     if (height > 0)
 	W_ResizeView(bPtr->menuView, bPtr->view->size.width, height);
 }
@@ -581,12 +507,15 @@ popUpMenu(PopUpButton *bPtr)
     Window dummyW;
     int x, y;
 
+    if (!bPtr->flags.enabled)
+	return;
+    
     if (!bPtr->menuView->flags.realized) {
 	W_RealizeView(bPtr->menuView);
 	resizeMenu(bPtr);
     }
     
-    if (bPtr->itemCount < 1)
+    if (WMGetArrayItemCount(bPtr->items) < 1)
 	return;
     
     XTranslateCoordinates(scr->display, bPtr->view->window, scr->rootWin,
@@ -603,10 +532,13 @@ popUpMenu(PopUpButton *bPtr)
 			       makeMenuPixmap(bPtr));
     XClearWindow(scr->display, bPtr->menuView->window);
     
+    if (W_VIEW_WIDTH(bPtr->menuView) != W_VIEW_WIDTH(bPtr->view))
+	resizeMenu(bPtr);
+    
     W_MapView(bPtr->menuView);
     
     bPtr->highlightedItem = 0;
-    if (!bPtr->flags.pullsDown && bPtr->selectedItem != NULL)
+    if (!bPtr->flags.pullsDown && bPtr->selectedItemIndex < 0)
 	paintMenuEntry(bPtr, bPtr->selectedItemIndex, True);
 }
 
@@ -615,22 +547,6 @@ static void
 popDownMenu(PopUpButton *bPtr)
 {
     W_UnmapView(bPtr->menuView);
-    
-    /* free the background pixmap used to draw the menu contents */
-    XSetWindowBackgroundPixmap(bPtr->view->screen->display, 
-			       bPtr->menuView->window, None);
-}
-
-
-static int
-itemIsEnabled(PopUpButton *bPtr, int index)
-{
-    ItemList *item = bPtr->items;
-     
-    while (index-- > 0 && item)
-	item = item->nextPtr;
-
-    return item ? !item->disabled : False;
 }
 
 
@@ -674,9 +590,18 @@ autoScroll(void *data)
 	    / bPtr->view->size.height;
 
 	if (oldItem!=bPtr->highlightedItem) {
+            WMMenuItem *item;
+
 	    paintMenuEntry(bPtr, oldItem, False);
-	    paintMenuEntry(bPtr, bPtr->highlightedItem,
-			   itemIsEnabled(bPtr, bPtr->highlightedItem));
+
+            if (bPtr->highlightedItem >= 0 &&
+                bPtr->highlightedItem < WMGetArrayItemCount(bPtr->items)) {
+                item = WMGetPopUpButtonMenuItem(bPtr, bPtr->highlightedItem);
+                paintMenuEntry(bPtr, bPtr->highlightedItem,
+                               WMGetMenuItemEnabled(item));
+            } else {
+                bPtr->highlightedItem = -1;
+            }
 	}
 
 	bPtr->timer = WMAddTimerHandler(SCROLL_DELAY, autoScroll, bPtr);
@@ -686,6 +611,37 @@ autoScroll(void *data)
 }
 
 
+static void
+wheelScrollUp(PopUpButton *bPtr)
+{
+    int testIndex = bPtr->selectedItemIndex - 1;
+
+    while (testIndex>=0 && !WMGetPopUpButtonItemEnabled(bPtr, testIndex))
+        testIndex--;
+    if (testIndex != -1) {
+	WMSetPopUpButtonSelectedItem(bPtr, testIndex);
+        if (bPtr->action)
+            (*bPtr->action)(bPtr, bPtr->clientData);
+    }
+}
+
+
+static void
+wheelScrollDown(PopUpButton *bPtr)
+{
+    int itemCount = WMGetArrayItemCount(bPtr->items);
+    int testIndex = bPtr->selectedItemIndex + 1;
+
+    while (testIndex<itemCount && !WMGetPopUpButtonItemEnabled(bPtr, testIndex))
+	testIndex++;
+    if (testIndex != itemCount) {
+	WMSetPopUpButtonSelectedItem(bPtr, testIndex);
+        if (bPtr->action)
+            (*bPtr->action)(bPtr, bPtr->clientData);
+    }
+}
+
+
 static void
 handleActionEvents(XEvent *event, void *data)
 {
@@ -695,14 +651,13 @@ handleActionEvents(XEvent *event, void *data)
 
     CHECK_CLASS(data, WC_PopUpButton);
 
-    if (bPtr->itemCount < 1)
+    if (WMGetArrayItemCount(bPtr->items) < 1)
 	return;
     
     switch (event->type) {
 	/* called for menuView */
      case Expose:
-	paintMenuEntry(bPtr, bPtr->highlightedItem,
-		       itemIsEnabled(bPtr, bPtr->highlightedItem));
+	paintMenuEntry(bPtr, bPtr->highlightedItem, True);
 	break;
 
      case LeaveNotify:
@@ -721,9 +676,18 @@ handleActionEvents(XEvent *event, void *data)
 	    oldItem = bPtr->highlightedItem;
 	    bPtr->highlightedItem = event->xmotion.y / bPtr->view->size.height;
 	    if (oldItem!=bPtr->highlightedItem) {
+		WMMenuItem *item;
+		
 		paintMenuEntry(bPtr, oldItem, False);
-		paintMenuEntry(bPtr, bPtr->highlightedItem, 
-			       itemIsEnabled(bPtr, bPtr->highlightedItem));
+                if (bPtr->highlightedItem >= 0 &&
+                    bPtr->highlightedItem < WMGetArrayItemCount(bPtr->items)) {
+                    item = WMGetPopUpButtonMenuItem(bPtr, bPtr->highlightedItem);
+                    paintMenuEntry(bPtr, bPtr->highlightedItem,
+                                   WMGetMenuItemEnabled(item));
+                } else {
+                    bPtr->highlightedItem = -1;
+                }
+
 	    }
 
 	    if (event->xmotion.y_root >= scrHeight-1
@@ -743,7 +707,18 @@ handleActionEvents(XEvent *event, void *data)
 	if (!bPtr->flags.enabled)
 	    break;
 
-	popUpMenu(bPtr);
+        if (event->xbutton.button==WINGsConfiguration.mouseWheelUp) {
+            if (!bPtr->menuView->flags.mapped && !bPtr->flags.pullsDown) {
+                wheelScrollUp(bPtr);
+            }
+            break;
+        } else if (event->xbutton.button==WINGsConfiguration.mouseWheelDown) {
+            if (!bPtr->menuView->flags.mapped && !bPtr->flags.pullsDown) {
+                wheelScrollDown(bPtr);
+            }
+            break;
+        }
+        popUpMenu(bPtr);
 	if (!bPtr->flags.pullsDown) {
 	    bPtr->highlightedItem = bPtr->selectedItemIndex;
 	    bPtr->flags.insideMenu = 1;
@@ -758,6 +733,10 @@ handleActionEvents(XEvent *event, void *data)
 	break;
  
      case ButtonRelease:
+	if (event->xbutton.button==WINGsConfiguration.mouseWheelUp ||
+            event->xbutton.button==WINGsConfiguration.mouseWheelDown) {
+            break;
+        }
 	XUngrabPointer(bPtr->view->screen->display, event->xbutton.time);
 	if (!bPtr->flags.pullsDown)
 	    popDownMenu(bPtr);
@@ -767,8 +746,12 @@ handleActionEvents(XEvent *event, void *data)
 	    bPtr->timer = NULL;
 	}
 
-	if (bPtr->flags.insideMenu && bPtr->highlightedItem>=0) {
-            if (itemIsEnabled(bPtr, bPtr->highlightedItem)) {
+	if (bPtr->flags.insideMenu && bPtr->highlightedItem>=0) {	
+	    WMMenuItem *item;
+
+	    item = WMGetPopUpButtonMenuItem(bPtr, bPtr->highlightedItem);
+
+            if (WMGetMenuItemEnabled(item)) {
 		int i;
                 WMSetPopUpButtonSelectedItem(bPtr, bPtr->highlightedItem);
 
@@ -799,25 +782,17 @@ handleActionEvents(XEvent *event, void *data)
 static void
 destroyPopUpButton(PopUpButton *bPtr)
 {
-    ItemList *itemPtr, *tmp;
-
     if (bPtr->timer) {
 	WMDeleteTimerHandler(bPtr->timer);
     }
-    
-    itemPtr = bPtr->items;
-    while (itemPtr!=NULL) {
-	free(itemPtr->text);
-	tmp = itemPtr->nextPtr;
-	free(itemPtr);
-	itemPtr = tmp;
-    }
-    
+
+    WMFreeArray(bPtr->items);
+
     if (bPtr->caption)
-	free(bPtr->caption);
+	wfree(bPtr->caption);
 
     /* have to destroy explicitly because the popup is a toplevel */
     W_DestroyView(bPtr->menuView);
 
-    free(bPtr);
+    wfree(bPtr);
 }
