@@ -78,7 +78,7 @@ static struct {
     {SHADE_STEPS_F, SHADE_DELAY_F},
     {SHADE_STEPS_M, SHADE_DELAY_M},
     {SHADE_STEPS_S, SHADE_DELAY_S},
-    {SHADE_STEPS_U, SHADE_DELAY_U}};
+    {SHADE_STEPS_US, SHADE_DELAY_US}};
 
 #define SHADE_STEPS	shadePars[(int)wPreferences.shade_speed].steps
 #define SHADE_DELAY	shadePars[(int)wPreferences.shade_speed].delay
@@ -255,7 +255,7 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 void
 wShadeWindow(WWindow  *wwin)
 {
-    time_t time0 = time(NULL);
+    time_t time0;
 #ifdef ANIMATIONS
     int y, s, w, h;
 #endif
@@ -263,6 +263,8 @@ wShadeWindow(WWindow  *wwin)
     if (wwin->flags.shaded)
 	return;
 
+    time0 = time(NULL);
+
     XLowerWindow(dpy, wwin->client_win);    
 
     wSoundPlay(WSOUND_SHADE);
@@ -284,8 +286,11 @@ wShadeWindow(WWindow  *wwin)
 	    if (time(NULL)-time0 > MAX_ANIMATION_TIME)
 		break;
 
-            if (SHADE_DELAY > 0)
+            if (SHADE_DELAY > 0) {
                 wusleep(SHADE_DELAY*1000L);
+            } else {
+                wusleep(10);
+            }
 	    h-=s;
 	    y-=s;
 	}
@@ -329,7 +334,7 @@ wShadeWindow(WWindow  *wwin)
 void
 wUnshadeWindow(WWindow  *wwin)
 {
-    time_t time0 = time(NULL);
+    time_t time0;
 #ifdef ANIMATIONS
     int y, s, w, h;    
 #endif /* ANIMATIONS */
@@ -337,7 +342,9 @@ wUnshadeWindow(WWindow  *wwin)
 
     if (!wwin->flags.shaded)
 	return;
-    
+
+    time0 = time(NULL);
+
     wwin->flags.shaded = 0;
     wwin->flags.mapped = 1;
     XMapWindow(dpy, wwin->client_win);
@@ -359,8 +366,11 @@ wUnshadeWindow(WWindow  *wwin)
 		XResizeWindow(dpy, wwin->frame->core->window, w, h);
 		XMoveWindow(dpy, wwin->client_win, 0, y);
 		XFlush(dpy);
-                if (SHADE_DELAY > 0)
+                if (SHADE_DELAY > 0) {
                     wusleep(SHADE_DELAY*2000L/3);
+                } else {
+                    wusleep(10);
+                }
 		h+=s;
 		y+=s;
 		
@@ -557,6 +567,8 @@ animateResizeFlip(WScreen *scr, int x, int y, int w, int h,
 	XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY_F > 0)
         wusleep(MINIATURIZE_ANIMATION_DELAY_F);
+#else
+        wusleep(10);
 #endif
 
 	XDrawLines(dpy,scr->root_win,scr->frame_gc,points, 5, CoordModeOrigin);
@@ -623,6 +635,8 @@ animateResizeTwist(WScreen *scr, int x, int y, int w, int h,
 	XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY_T > 0)
         wusleep(MINIATURIZE_ANIMATION_DELAY_T);
+#else
+        wusleep(10);
 #endif
 
         XDrawLines(dpy, scr->root_win, scr->frame_gc, points, 5, CoordModeOrigin);
@@ -669,6 +683,8 @@ animateResizeZoom(WScreen *scr, int x, int y, int w, int h,
 	XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY_Z > 0)
         wusleep(MINIATURIZE_ANIMATION_DELAY_Z);
+#else
+        wusleep(10);
 #endif
 	for (j=0; j<FRAMES; j++) {
 	    XDrawRectangle(dpy, scr->root_win, scr->frame_gc, 
@@ -694,6 +710,8 @@ animateResizeZoom(WScreen *scr, int x, int y, int w, int h,
     XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY_Z > 0)
     wusleep(MINIATURIZE_ANIMATION_DELAY_Z);
+#else
+    wusleep(10);
 #endif
     for (j=0; j<FRAMES; j++) {
 	XDrawRectangle(dpy, scr->root_win, scr->frame_gc, 
@@ -1137,6 +1155,9 @@ wDeiconifyWindow(WWindow  *wwin)
     }
 
     WMPostNotificationName(WMNChangedState, wwin, "iconify");
+
+    /* In case we were shaded and iconified, also unshade */
+    wUnshadeWindow(wwin);
 }
 
 
@@ -1305,8 +1326,7 @@ unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate,
 	wWindowChangeWorkspace(wwin, wwin->screen_ptr->current_workspace);
 
     wwin->flags.hidden=0;
-    wwin->flags.mapped=1;
-    
+
     wSoundPlay(WSOUND_UNHIDE);
 #ifdef ANIMATIONS
     if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations 
@@ -1319,12 +1339,15 @@ unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate,
     }
 #endif
     wwin->flags.skip_next_animation = 0;
-    XMapWindow(dpy, wwin->client_win);
-    XMapWindow(dpy, wwin->frame->core->window);
-    wClientSetState(wwin, NormalState, None);
-    wRaiseFrame(wwin->frame->core);
+    if (wwin->screen_ptr->current_workspace == wwin->frame->workspace) {
+        XMapWindow(dpy, wwin->client_win);
+        XMapWindow(dpy, wwin->frame->core->window);
+        wClientSetState(wwin, NormalState, None);
+        wwin->flags.mapped=1;
+        wRaiseFrame(wwin->frame->core);
+    }
     if (wwin->flags.inspector_open) {
-	wUnhideInspectorForWindow(wwin);
+        wUnhideInspectorForWindow(wwin);
     }
 
     WMPostNotificationName(WMNChangedState, wwin, "hide");
@@ -1531,7 +1554,7 @@ wArrangeIcons(WScreen *scr, Bool arrangeAll)
     pi = 0;
     si = 0;
     while (aicon) {	
-        if (!aicon->docked && wAppIconIsFirstInstance(aicon)) {
+        if (!aicon->docked) {
             if (aicon->x_pos != X || aicon->y_pos != Y) {
 #ifdef ANIMATIONS
                 if (!wPreferences.no_animations) {
