@@ -436,7 +436,12 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		i = 0;
 	    }
 	}
-	
+
+        /* Handling this may require more work. -Dan */
+        if (wwin->fake_group!=NULL) {
+            i = 7;
+        }
+
 	if (wwin->wm_hints)
 	  XFree(wwin->wm_hints);
 
@@ -475,7 +480,10 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	    wwin->group_id = wwin->main_window;
 	    wApplicationCreate(wwin->screen_ptr, wwin->main_window);
 	    break;
-	}
+         /* 7 - we have a fake window group id, so just ignore anything else */
+         case 7:
+            break;
+        }
 #ifdef DEBUG
 	if (i) {
 	    printf("window leader update caused state transition %i\n",i);
@@ -504,8 +512,10 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	      wwin->flags.urgent = 1;
 	    else
 	      wwin->flags.urgent = 0;
-	} else {
-	    wwin->group_id = None;
+        /*} else if (wwin->fake_group!=NULL) {
+            wwin->group_id = wwin->fake_group->leader;*/
+        } else {
+            wwin->group_id = None;
 	}
 	break;
 	
@@ -597,11 +607,51 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		    /* TODO: remake appmenu update */
 		    wAppMenuDestroy(wapp->menu);
 		}
-		wapp->menu = wAppMenuGet(wwin->screen_ptr, wwin->main_window);
-		/* make the appmenu be mapped */
-		wSetFocusTo(wwin->screen_ptr, NULL);
-		wSetFocusTo(wwin->screen_ptr, 
-			    wwin->screen_ptr->focused_window);
+                if (wwin->fake_group) {
+                    extern WPreferences wPreferences;
+                    WScreen *scr = wwin->screen_ptr;
+                    WWindow *foo = scr->focused_window;
+                    WFakeGroupLeader *fPtr = wwin->fake_group;
+
+                    wApplicationDestroy(wapp);
+                    while (foo) {
+                        if (foo->fake_group && foo->fake_group==fPtr) {
+                            WSETUFLAG(foo, shared_appicon, 0);
+                            foo->fake_group = NULL;
+                            if (foo->group_id!=None)
+                                foo->main_window = foo->group_id;
+                            else if (foo->client_leader!=None)
+                                foo->main_window = foo->client_leader;
+                            else if (WFLAGP(foo, emulate_appicon))
+                                foo->main_window = foo->client_win;
+                            else
+                                foo->main_window = None;
+                            if (foo->main_window) {
+                                wapp = wApplicationCreate(scr, foo->main_window);
+                            }
+                        }
+                        foo = foo->prev;
+                    }
+
+                    if (fPtr->leader!=None)
+                        XDestroyWindow(dpy, fPtr->leader);
+                    fPtr->retainCount = 0;
+                    fPtr->leader = None;
+                    fPtr->origLeader = None;
+
+                    wapp = wApplicationOf(wwin->main_window);
+                    if (wapp) {
+                        wapp->menu = wAppMenuGet(scr, wwin->main_window);
+                    }
+                    if (wPreferences.auto_arrange_icons) {
+                        wArrangeIcons(wwin->screen_ptr, True);
+                    }
+                } else {
+                    wapp->menu = wAppMenuGet(wwin->screen_ptr, wwin->main_window);
+                }
+                /* make the appmenu be mapped */
+                wSetFocusTo(wwin->screen_ptr, NULL);
+                wSetFocusTo(wwin->screen_ptr, wwin->screen_ptr->focused_window);
 	    }
 	} else if (event->atom==_XA_GNUSTEP_WM_ATTR) {
 	    GNUstepWMAttributes *attr;
