@@ -107,6 +107,64 @@ wMessageDialog(WScreen *scr, char *title, char *message,
 }
 
 
+void
+toggleSaveSession(WMWidget *w, void *data)
+{
+    wPreferences.save_session_on_exit = WMGetButtonSelected((WMButton *) w);
+}
+
+
+int
+wExitDialog(WScreen *scr, char *title, char *message,
+            char *defBtn, char *altBtn, char *othBtn)
+{
+    WMAlertPanel *panel;
+    WMButton *saveSessionBtn;
+    Window parent;
+    WWindow *wwin;
+    int result;
+
+    panel = WMCreateAlertPanel(scr->wmscreen, NULL, title, message,
+                               defBtn, altBtn, othBtn);
+
+    /* add save session button */
+    saveSessionBtn = WMCreateSwitchButton(panel->hbox);
+    WMSetButtonAction(saveSessionBtn, toggleSaveSession, NULL);
+    WMAddBoxSubview(panel->hbox, WMWidgetView(saveSessionBtn),
+                            False, True, 200, 0, 0);
+    WMSetButtonText(saveSessionBtn, _("Save workspace state"));
+    WMSetButtonSelected(saveSessionBtn, wPreferences.save_session_on_exit);
+    WMRealizeWidget(saveSessionBtn);
+    WMMapWidget(saveSessionBtn);
+
+    parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 400, 180, 0, 0, 0);
+
+    XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
+
+    wwin = wManageInternalWindow(scr, parent, None, NULL,
+                                (scr->scr_width - 400)/2,
+                                (scr->scr_height - 180)/2, 400, 180);
+    wwin->client_leader = WMWidgetXID(panel->win);
+
+    WMMapWidget(panel->win);
+
+    wWindowMap(wwin);
+
+    WMRunModalLoop(WMWidgetScreen(panel->win), WMWidgetView(panel->win));
+
+    result = panel->result;
+
+    WMUnmapWidget(panel->win);
+
+    wUnmanageWindow(wwin, False, False);
+
+    WMDestroyAlertPanel(panel);
+
+    XDestroyWindow(dpy, parent);
+
+    return result;
+}
+
 
 int
 wInputDialog(WScreen *scr, char *title, char *message, char **text)
@@ -903,28 +961,50 @@ logoPushCallback(void *data)
     static int oldi = 0;
     int len;
     static int jingobeu[] = {
-	329, 150, -1, 100, 329, 150, -1, 100, 329, 300, -1, 250,
-	    329, 150, -1, 100, 329, 150, -1, 100,  329, 300, -1, 250,
-	    329, 150, 392, 150, 261, 150, 293, 150, 329, 400, -1, 400, 0
+        329, 150, -1, 100, 329, 150, -1, 100, 329, 300, -1, 250,
+        329, 150, -1, 100, 329, 150, -1, 100,  329, 300, -1, 250,
+        329, 150, 392, 150, 261, 150, 293, 150, 329, 400, -1, 400, 0
     };
     static int c = 0;
 
     if (panel->x) {
-	XKeyboardControl kc;
+        XKeyboardControl kc;
+        XKeyboardState ksave;
+        unsigned long mask = KBBellPitch|KBBellDuration|KBBellPercent;
+
+        XGetKeyboardControl(dpy, &ksave);
 
 	if (panel->x > 0) {
-	    if(jingobeu[panel->x-1]==0){panel->x=-1;}else if(jingobeu[panel->x
--1]<0){panel->x++;c=jingobeu[panel->x-1]/50;panel->x++;}else if(c==0){
-    kc.bell_pitch=jingobeu[panel->x-1];panel->x++;kc.bell_percent=50;c=
-	jingobeu[panel->x-1]/50;kc.bell_duration=jingobeu[panel->x-1];panel->x++;
-XChangeKeyboardControl(dpy,KBBellPitch|KBBellDuration|KBBellPercent,&kc);
-    XBell(dpy,50);XFlush(dpy);}else{c--;}}
+            if(jingobeu[panel->x-1]==0) {
+                panel->x=-1;
+            } else if (jingobeu[panel->x-1]<0) {
+                panel->x++;
+                c=jingobeu[panel->x-1]/50;
+                panel->x++;
+            } else if (c==0) {
+                kc.bell_percent=50;
+                kc.bell_pitch=jingobeu[panel->x-1];
+                panel->x++;
+                kc.bell_duration=jingobeu[panel->x-1];
+                c=jingobeu[panel->x-1]/50;
+                panel->x++;
+                XChangeKeyboardControl(dpy, mask, &kc);
+                XBell(dpy, 50);
+                XFlush(dpy);
+            } else {
+                c--;
+            }
+        }
 	if (!(panel->cycle % 4)) {
 	    WMPixmap *p;
 
 	    p = DoXThing(panel->wwin);
 	    WMSetLabelImage(panel->logoL, p);
-	}
+        }
+        kc.bell_pitch = ksave.bell_pitch;
+        kc.bell_percent = ksave.bell_percent;
+        kc.bell_duration = ksave.bell_duration;
+        XChangeKeyboardControl(dpy, mask, &kc);
     } else if (panel->cycle < 30) {
 	RImage *image;
         WMPixmap *pix;
@@ -1085,8 +1165,9 @@ handleLogoPush(XEvent *event, void *data)
         "Mooo Canada!!!!",
         "Hi! My name is bobby...",
         "AHH! The neurotic monkeys are after me!",
-        "WHAT YOU SAY??",
         "WE GET SIGNAL",
+        "HOW ARE YOU GENTLEMEN?",
+        "WHAT YOU SAY??",
         "SOMEBODY SET UP US THE BOMB",
         "ALL YOUR BASE ARE BELONG TO US!",
         "Oh My God!!! Larry is back!"
@@ -1411,7 +1492,7 @@ wShowInfoPanel(WScreen *scr)
 	panel->timer = WMAddTimerHandler(100, logoPushCallback, panel);
 	panel->cycle = 0;
 	panel->x = 1;
-	panel->str = _("Merry X'mas!");
+	panel->str = _("Merry Christmas!");
 	panel->oldPix = WMRetainPixmap(WMGetLabelImage(panel->logoL));
     }
 #endif
