@@ -24,14 +24,29 @@
 #define WMDIALOG_H_
 
 
+enum {
+    WMAbort=0,
+    WMRestart,
+    WMStartAlternate
+};
+
+
 int wMessageDialog(WScreen *scr, char *title, char *message, 
 		   char *defBtn, char *altBtn, char *othBtn);
 int wInputDialog(WScreen *scr, char *title, char *message, char **text);
 
+int wExitDialog(WScreen *scr, char *title, char *message, char *defBtn,
+                char *altBtn, char *othBtn);
+
 Bool wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class);
 
 void wShowInfoPanel(WScreen *scr);
 
 void wShowLegalPanel(WScreen *scr);
 
+void wShowGNUstepPanel(WScreen *scr);
+
+int wShowCrashingDialogPanel(int whatSig);
+
+
 #endif
