@@ -82,9 +82,6 @@ typedef struct WFrameWindow {
     union WTexture **title_texture;
     union WTexture **resizebar_texture;
     unsigned long *title_pixel;
-#ifdef DRAWSTRING_PLUGIN
-    int drawstring_proc_offset;
-#endif
     GC *title_gc;
     WMFont **font;
 
@@ -161,10 +158,7 @@ wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y,
 		   union WTexture **title_texture, 
 		   union WTexture **resize_texture,
 		   unsigned long *color, 
-#ifdef DRAWSTRING_PLUGIN
-           int function_offset,
-#endif
-           GC *gc, WMFont **font);
+		   GC *gc, WMFont **font);
 
 void wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags);
 
