@@ -26,6 +26,7 @@
 #include <stdio.h>
 
 #include "window.h"
+#include "defaults.h"
 
 typedef void (WCallBack)(void *cdata);
 
@@ -124,6 +125,8 @@ char *EscapeWM_CLASS(char *name, char *class);
 
 void UnescapeWM_CLASS(char *str, char **name, char **class);
 
+Bool UpdateDomainFile(WDDomain *domain);
+
 #ifdef NUMLOCK_HACK
 void wHackedGrabKey(int keycode, unsigned int modifiers,
 		    Window grab_window, Bool owner_events, int pointer_mode,
