@@ -350,7 +350,7 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 	{ICON_SLIDE_DELAY_F, ICON_SLIDE_STEPS_F, ICON_SLIDE_SLOWDOWN_F},
 	{ICON_SLIDE_DELAY_M, ICON_SLIDE_STEPS_M, ICON_SLIDE_SLOWDOWN_M},
 	{ICON_SLIDE_DELAY_S, ICON_SLIDE_STEPS_S, ICON_SLIDE_SLOWDOWN_S},
-	{ICON_SLIDE_DELAY_U, ICON_SLIDE_STEPS_U, ICON_SLIDE_SLOWDOWN_U}};
+	{ICON_SLIDE_DELAY_US, ICON_SLIDE_STEPS_US, ICON_SLIDE_SLOWDOWN_US}};
     
     
 
@@ -407,6 +407,8 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
         XFlush(dpy);
         if (apars[(int)wPreferences.icon_slide_speed].delay > 0) {
             wusleep(apars[(int)wPreferences.icon_slide_speed].delay*1000L);
+        } else {
+            wusleep(10);
         }
 	if (time(NULL) - time0 > MAX_ANIMATION_TIME)
 	    break;
@@ -1234,3 +1236,38 @@ SendHelperMessage(WScreen *scr, char type, int workspace, char *msg)
     }
     wfree(buffer);
 }
+
+
+Bool
+UpdateDomainFile(WDDomain *domain)
+{
+    struct stat stbuf;
+    char path[PATH_MAX];
+    WMPropList *shared_dict=NULL, *dict;
+    Bool result, freeDict = False;
+
+    dict = domain->dictionary;
+    if (WMIsPLDictionary(domain->dictionary)) {
+        /* retrieve global system dictionary */
+        snprintf(path, sizeof(path), "%s/WindowMaker/%s",
+                 SYSCONFDIR, domain->domain_name);
+        if (stat(path, &stbuf) >= 0) {
+            shared_dict = WMReadPropListFromFile(path);
+            if (shared_dict && WMIsPLDictionary(shared_dict)) {
+                freeDict = True;
+                dict = WMDeepCopyPropList(domain->dictionary);
+                WMSubtractPLDictionaries(dict, shared_dict, True);
+            }
+        }
+    }
+
+    result = WMWritePropListToFile(dict, domain->path, True);
+
+    if (freeDict) {
+        WMReleasePropList(dict);
+    }
+
+    return result;
+}
+
+
