@@ -128,6 +128,7 @@ static Shortcut *shortcutList = NULL;
  *                the resulting menu in current position. The output of
  *                command must be a valid menu description.
  *                The space between '|' and command is optional.
+ *                || will do the same, but will not cache the contents.
  * SAVE_SESSION - saves the current state of the desktop, which include
  *		  all running applications, all their hints (geometry,
  *		  position on screen, workspace they live on, the dock
@@ -174,21 +175,44 @@ static void
 exitCommand(WMenu *menu, WMenuEntry *entry)
 {
     static int inside = 0;
+    int result;
 
     /* prevent reentrant calls */
     if (inside)
 	return;
     inside = 1;
 
-    if ((long)entry->clientdata==M_QUICK
-	|| wMessageDialog(menu->frame->screen_ptr, _("Exit"),
-			  _("Exit window manager?"), 
-			  _("Exit"), _("Cancel"), NULL)==WAPRDefault) {
+#define R_CANCEL 0
+#define R_EXIT   1
+
+    result = R_CANCEL;
+
+    if ((long)entry->clientdata==M_QUICK) {
+        result = R_EXIT;
+    } else {
+        int r, oldSaveSessionFlag;
+
+        oldSaveSessionFlag = wPreferences.save_session_on_exit;
+        r = wExitDialog(menu->frame->screen_ptr, _("Exit"),
+                        _("Exit window manager?"),
+                        _("Exit"), _("Cancel"), NULL);
+
+        if (r==WAPRDefault) {
+            result = R_EXIT;
+        } else if (r==WAPRAlternate) {
+            /* Don't modify the "save session on exit" flag if the
+             * user canceled the operation. */
+            wPreferences.save_session_on_exit = oldSaveSessionFlag;
+        }
+    }
+    if (result==R_EXIT) {
 #ifdef DEBUG
 	printf("Exiting WindowMaker.\n");
 #endif
 	Shutdown(WSExitMode);
     }
+#undef R_EXIT
+#undef R_CANCEL
     inside = 0;
 }
 
@@ -229,15 +253,22 @@ shutdownCommand(WMenu *menu, WMenuEntry *entry)
 	} else
 #endif
 	{
-	    int r;
+	    int r, oldSaveSessionFlag;
 
-	    r = wMessageDialog(menu->frame->screen_ptr, 
-			       _("Kill X session"), 
-			       _("Kill Window System session?\n"
-				 "(all applications will be closed)"),
-			       _("Kill"), _("Cancel"), NULL);
-	    if (r==WAPRDefault)
-		result = R_KILL;
+            oldSaveSessionFlag = wPreferences.save_session_on_exit;
+
+	    r = wExitDialog(menu->frame->screen_ptr,
+                            _("Kill X session"),
+                            _("Kill Window System session?\n"
+                              "(all applications will be closed)"),
+                            _("Kill"), _("Cancel"), NULL);
+            if (r==WAPRDefault) {
+                result = R_KILL;
+            } else if (r==WAPRAlternate) {
+                /* Don't modify the "save session on exit" flag if the
+                 * user canceled the operation. */
+                wPreferences.save_session_on_exit = oldSaveSessionFlag;
+            }
 	}
     } 
     
@@ -649,19 +680,22 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
 		 (char*)entry->clientdata);
 	return;
     }
-    
+
     if (path[0][0]=='|') {
         /* pipe menu */
-	
+
         if (!menu->cascades[entry->cascade] || 
             menu->cascades[entry->cascade]->timestamp == 0) { 
             /* parse pipe */
 
 	    submenu = readMenuPipe(menu->frame->screen_ptr, path);
 
-            /* there's no automatic reloading */
-            if(submenu != NULL)
-                submenu->timestamp = 1;
+            if(submenu != NULL) {
+		if (path[0][1] == '|')
+		    submenu->timestamp = 0;
+		else
+                    submenu->timestamp = 1; /* there's no automatic reloading */
+	    }
         } else { 
             submenu = NULL; 
         }
