@@ -660,6 +660,8 @@ wScreenInit(int screen_number)
     scr->totalUsableArea.x2 = scr->scr_width;
     scr->totalUsableArea.y2 = scr->scr_height;
 
+    scr->fakeGroupLeaders = WMCreateArray(16);
+
 #if 0
     if (!aquireManagerSelection(scr)) {
 	wfree(scr);
@@ -697,9 +699,7 @@ wScreenInit(int screen_number)
 	return NULL;
     }
 
-#ifndef DEFINABLE_CURSOR
-    XDefineCursor(dpy, scr->root_win, wCursor[WCUR_DEFAULT]);
-#endif
+    XDefineCursor(dpy, scr->root_win, wCursor[WCUR_ROOT]);
 
     /* screen descriptor for raster graphic library */
     rattr.flags = RC_RenderMode | RC_ColorsPerChannel | RC_StandardColormap;
