@@ -24,6 +24,12 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
+#ifdef TEXTURE_PLUGIN
+# ifdef HAVE_DLFCN_H
+#  include <dlfcn.h>
+# endif
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -38,10 +44,6 @@
 extern WPreferences wPreferences;
 
 
-static Pixmap renderTexture(WScreen *scr, int width, int height,
-			     WTexture *texture, int rel);
-
-
 static void bevelImage(RImage *image, int relief);
 
 
@@ -141,7 +143,7 @@ wTextureDestroy(WScreen *scr, WTexture *texture)
 	return;
     }
 #endif
-    
+
     /* 
      * some stupid servers don't like white or black being freed...
      */
@@ -160,23 +162,37 @@ wTextureDestroy(WScreen *scr, WTexture *texture)
 	break;
 
      case WTEX_PIXMAP:
-	RDestroyImage(texture->pixmap.pixmap);
+	RReleaseImage(texture->pixmap.pixmap);
 	break;
 	
      case WTEX_MHGRADIENT:
      case WTEX_MVGRADIENT:
      case WTEX_MDGRADIENT:
 	for (i=0; texture->mgradient.colors[i]!=NULL; i++) {
-	    free(texture->mgradient.colors[i]);
+	    wfree(texture->mgradient.colors[i]);
 	}
-	free(texture->mgradient.colors);
+	wfree(texture->mgradient.colors);
 	break;
 
      case WTEX_THGRADIENT:
      case WTEX_TVGRADIENT:
      case WTEX_TDGRADIENT:	
-	RDestroyImage(texture->tgradient.pixmap);
+	RReleaseImage(texture->tgradient.pixmap);
 	break;
+
+#ifdef TEXTURE_PLUGIN
+     case WTEX_FUNCTION:
+#ifdef HAVE_DLFCN_H
+        if (texture->function.handle) {
+	    dlclose(texture->function.handle);
+	}
+#endif
+	for (i = 0; i < texture->function.argc; i++) {
+	    wfree(texture->function.argv[i]);
+	}
+	wfree(texture->function.argv);
+	break;
+#endif /* TEXTURE_PLUGIN */
     }
     if (CANFREE(texture->any.color.pixel))
       colors[count++] = texture->any.color.pixel;
@@ -192,7 +208,7 @@ wTextureDestroy(WScreen *scr, WTexture *texture)
 	XSetErrorHandler(oldhandler);
     }
     XFreeGC(dpy, texture->any.gc);
-    free(texture);
+    wfree(texture);
 #undef CANFREE    
 }
 
@@ -228,6 +244,45 @@ wTextureMakeGradient(WScreen *scr, int style, RColor *from, RColor *to)
 
 
 
+
+WTexIGradient*
+wTextureMakeIGradient(WScreen *scr, int thickness1, RColor colors1[2],
+		      int thickness2, RColor colors2[2])
+{
+    WTexIGradient *texture;
+    XGCValues gcv;
+    int i;
+
+    
+    texture = wmalloc(sizeof(WTexture));
+    memset(texture, 0, sizeof(WTexture));
+    texture->type = WTEX_IGRADIENT;
+    for (i = 0; i < 2; i++) {
+	texture->colors1[i] = colors1[i];
+	texture->colors2[i] = colors2[i];
+    }
+    texture->thickness1 = thickness1;
+    texture->thickness2 = thickness2;
+    if (thickness1 >= thickness2) {
+	texture->normal.red = (colors1[0].red + colors1[1].red)<<7;
+	texture->normal.green = (colors1[0].green + colors1[1].green)<<7;
+	texture->normal.blue = (colors1[0].blue + colors1[1].blue)<<7;
+    } else {
+	texture->normal.red = (colors2[0].red + colors2[1].red)<<7;
+	texture->normal.green = (colors2[0].green + colors2[1].green)<<7;
+	texture->normal.blue = (colors2[0].blue + colors2[1].blue)<<7;
+    }
+    XAllocColor(dpy, scr->w_colormap, &texture->normal);
+    gcv.background = gcv.foreground = texture->normal.pixel;
+    gcv.graphics_exposures = False;
+    texture->normal_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
+				   |GCGraphicsExposures, &gcv);
+
+    return texture;
+}
+
+
+
 WTexMGradient*
 wTextureMakeMGradient(WScreen *scr, int style, RColor **colors)
 {
@@ -279,10 +334,10 @@ wTextureMakePixmap(WScreen *scr, int style, char *pixmap_file, XColor *color)
     if (!image) {
 	wwarning(_("could not load texture pixmap \"%s\":%s"), file,
 			RMessageForError(RErrorCode));
-	free(file);
+	wfree(file);
 	return NULL;
     }
-    free(file);
+    wfree(file);
 
     texture = wmalloc(sizeof(WTexture));
     memset(texture, 0, sizeof(WTexture));
@@ -302,8 +357,6 @@ wTextureMakePixmap(WScreen *scr, int style, char *pixmap_file, XColor *color)
     return texture;
 }
 
-
-
 WTexTGradient*
 wTextureMakeTGradient(WScreen *scr, int style, RColor *from, RColor *to,
 		      char *pixmap_file, int opacity)
@@ -323,10 +376,10 @@ wTextureMakeTGradient(WScreen *scr, int style, RColor *from, RColor *to,
     if (!image) {
 	wwarning(_("could not load texture pixmap \"%s\":%s"), file,
 		 RMessageForError(RErrorCode));
-	free(file);
+	wfree(file);
 	return NULL;
     }
-    free(file);
+    wfree(file);
 
     texture = wmalloc(sizeof(WTexture));
     memset(texture, 0, sizeof(WTexture));
@@ -353,11 +406,64 @@ wTextureMakeTGradient(WScreen *scr, int style, RColor *from, RColor *to,
 }
 
 
+#ifdef TEXTURE_PLUGIN
+WTexFunction*
+wTextureMakeFunction(WScreen *scr, char *lib, char *func, int argc, char **argv)
+{
+    XColor fallbackColor;
+    XGCValues gcv;
+    WTexFunction *texture;
+    
+    texture = wmalloc(sizeof(WTexture));
+    texture->type = WTEX_FUNCTION;
+    texture->handle = NULL;
+    texture->render = 0;
+    texture->argc = argc;
+    texture->argv = argv;
+    
+    fallbackColor.red = 0x8000;
+    fallbackColor.green = 0x8000;
+    fallbackColor.blue = 0x8000;
+    
+    gcv.background = gcv.foreground = fallbackColor.pixel;
+    gcv.graphics_exposures = False;
+    texture->normal_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
+				   |GCGraphicsExposures, &gcv);
+    
+# ifdef HAVE_DLFCN_H
+    /* open the library */
+    texture->handle = dlopen(lib, RTLD_LAZY);
+    if (!texture->handle) {
+        wwarning(_("library \"%s\" cound not be opened."), lib);
+	wfree(argv);
+	wfree(texture);
+	return NULL;
+    }
+    
+    /* find the function */
+    texture->render = dlsym(texture->handle, func);
+    if (!texture->render) {
+        wwarning(_("function \"%s\" not found in library \"%s\""), func, lib);
+	wfree(argv);
+	dlclose(texture->handle);
+	wfree(texture);
+	return NULL;
+    }
+# else
+    wwarning(_("function textures not supported on this system, sorry."));
+# endif
+    
+    /* success! */
+    return texture;
+}
+#endif /* TEXTURE_PLUGIN */
+
 
 RImage*
-wTextureRenderImage(WTexture *texture, int width, int height, int relief)
+wTextureRenderImage(WTexture *texture, int width, int height,
+		    int relief)
 {
-    RImage *image;
+    RImage *image = NULL;
     RColor color1;
     int d;
     int subtype;
@@ -389,6 +495,14 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
         }
 	break;
 	
+     case WTEX_IGRADIENT:
+	image = RRenderInterwovenGradient(width, height,
+					  texture->igradient.colors1,
+					  texture->igradient.thickness1,
+					  texture->igradient.colors2,
+					  texture->igradient.thickness2);
+	break;
+
      case WTEX_HGRADIENT:
 	subtype = RGRD_HORIZONTAL;
 	goto render_gradient;
@@ -442,17 +556,32 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
 	    grad = RRenderGradient(width, height, &texture->tgradient.color1,
 				   &texture->tgradient.color2, subtype);
 	    if (!grad) {
-		RDestroyImage(image);
+		RReleaseImage(image);
 		image = NULL;
 		break;
 	    }
 
 	    RCombineImagesWithOpaqueness(image, grad,
 					 texture->tgradient.opacity);
-	    RDestroyImage(grad);
+	    RReleaseImage(grad);
 	}
 	break;
 
+#ifdef TEXTURE_PLUGIN
+     case WTEX_FUNCTION:
+#ifdef HAVE_DLFCN_H
+	if (texture->function.render) {
+            image = texture->function.render (
+	        texture->function.argc, texture->function.argv,
+		width, height, relief);
+	}
+#endif
+	if (!image) {
+	    RErrorCode = RERR_INTERNAL;
+	}
+        break;
+#endif /* TEXTURE_PLUGIN */
+
      default:
 	puts("ERROR in wTextureRenderImage()");
 	image = NULL;
@@ -460,8 +589,21 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
     }
 
     if (!image) {
+	RColor gray;
+
 	wwarning(_("could not render texture: %s"), RMessageForError(RErrorCode));
-	return None;
+
+	image = RCreateImage(width, height, False);
+	if (image == NULL) {
+	    wwarning(_("could not allocate image buffer"));
+	    return NULL;
+	}
+       
+	gray.red = 190;
+	gray.green = 190;
+	gray.blue = 190;
+        gray.alpha = 255;
+	RClearImage(image, &gray);
     }
 
 
@@ -502,31 +644,6 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
 }
 
 
-/* used only for menu entries */
-void
-wTextureRender(WScreen *scr, WTexture *texture, Pixmap *data, 
-	       int width, int height, int relief)
-{
-    if (!texture)
-      return;
-/*    
-    switch (texture->any.type) {
-     case WTEX_DGRADIENT:
-     case WTEX_VGRADIENT:
-     case WTEX_HGRADIENT:
-     case WTEX_MHGRADIENT:
-     case WTEX_MVGRADIENT:
-     case WTEX_MDGRADIENT:
-     case WTEX_PIXMAP:
-	*/
-	if (!*data) {
-	    *data = renderTexture(scr, width, height, texture, relief);
-	}/*
-	break;
-    }*/
-}
-
-
 
 static void
 bevelImage(RImage *image, int relief)
@@ -539,7 +656,9 @@ bevelImage(RImage *image, int relief)
      case WREL_MENUENTRY:
 	color.red = color.green = color.blue = 80;
 	color.alpha = 0;
+	/**/
 	ROperateLine(image, RAddOperation, 1, 0, width-2, 0, &color);
+	/**/
 
 	ROperateLine(image, RAddOperation, 0, 0, 0, height-1, &color);
 
@@ -548,38 +667,19 @@ bevelImage(RImage *image, int relief)
 	ROperateLine(image, RSubtractOperation, width-1, 0, width-1, 
 		     height-1, &color);
 
+	/**/
 	ROperateLine(image, RSubtractOperation, 1, height-2, width-2,
 		     height-2, &color);
 
 	color.red = color.green = color.blue = 0;
 	color.alpha = 255;
 	RDrawLine(image, 0, height-1, width-1, height-1, &color);
+	/**/
 	break;
 
     }
 }
  
-
-
-static Pixmap
-renderTexture(WScreen *scr, int width, int height, WTexture *texture,
-	       int rel)
-{
-    RImage *img;
-    Pixmap pix;
-
-    img = wTextureRenderImage(texture, width, height, rel);
-    
-    if (!img) {
-	wwarning(_("could not render texture: %s"), RMessageForError(RErrorCode));
-	return None;
-    }
-    if (!RConvertImage(scr->rcontext, img, &pix)) {
-	wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
-    }
-    RDestroyImage(img);
-    return pix;
-}
  
 
 void
