@@ -80,11 +80,11 @@ static WMPropList *AUnfocusable;
 static WMPropList *AAlwaysUserIcon;
 static WMPropList *AStartMiniaturized;
 static WMPropList *AStartMaximized;
-static WMPropList *AStartHidden;	       /* app */
-static WMPropList *ADontSaveSession;    /* app */
+static WMPropList *AStartHidden;       /* app */
+static WMPropList *ADontSaveSession;   /* app */
 static WMPropList *AEmulateAppIcon;
 static WMPropList *AFullMaximize;
-static WMPropList *ACollapseAppIcons;   /* app */
+static WMPropList *ASharedAppIcon;     /* app */
 #ifdef XKB_BUTTON_HINT
 static WMPropList *ANoLanguageButton;
 #endif
@@ -125,7 +125,7 @@ init_wdefaults(WScreen *scr)
     ADontSaveSession = WMCreatePLString("DontSaveSession");
     AEmulateAppIcon = WMCreatePLString("EmulateAppIcon");
     AFullMaximize = WMCreatePLString("FullMaximize");
-    ACollapseAppIcons = WMCreatePLString("CollapseAppIcons");
+    ASharedAppIcon = WMCreatePLString("SharedAppIcon");
 #ifdef XKB_BUTTON_HINT
     ANoLanguageButton = WMCreatePLString("NoLanguageButton");
 #endif
@@ -202,12 +202,15 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
 
 
     if (class && instance) {
-      char *buffer = NULL;
-	  buffer = wmalloc(strlen(class)+strlen(instance)+4);
-      key1 = WMCreatePLString(strcat(strcat(strcpy(buffer,instance),"."),class));
-	  wfree(buffer);
-    } else
-      key1 = NULL;
+        char *buffer;
+
+        buffer = wmalloc(strlen(class)+strlen(instance)+2);
+        sprintf(buffer, "%s.%s", instance, class);
+        key1 = WMCreatePLString(buffer);
+        wfree(buffer);
+    } else {
+        key1 = NULL;
+    }
 
     if (instance)
       key2 = WMCreatePLString(instance);
@@ -253,13 +256,13 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
     /* get the data */        
     value = get_value(dw, dc, dn, da, ANoTitlebar, No, useGlobalDefault);
     APPLY_VAL(value, no_titlebar, ANoTitlebar);
-        
+
     value = get_value(dw, dc, dn, da, ANoResizebar, No, useGlobalDefault);
     APPLY_VAL(value, no_resizebar, ANoResizebar);
 
     value = get_value(dw, dc, dn, da, ANoMiniaturizeButton, No, useGlobalDefault);
     APPLY_VAL(value, no_miniaturize_button, ANoMiniaturizeButton);
-    
+
     value = get_value(dw, dc, dn, da, ANoCloseButton, No, useGlobalDefault);
     APPLY_VAL(value, no_close_button, ANoCloseButton);
 
@@ -268,19 +271,19 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
 
     value = get_value(dw, dc, dn, da, ANoHideOthers, No, useGlobalDefault);
     APPLY_VAL(value, no_hide_others, ANoHideOthers);
-    
+
     value = get_value(dw, dc, dn, da, ANoMouseBindings, No, useGlobalDefault);
     APPLY_VAL(value, no_bind_mouse, ANoMouseBindings);
-    
+
     value = get_value(dw, dc, dn, da, ANoKeyBindings, No, useGlobalDefault);
     APPLY_VAL(value, no_bind_keys, ANoKeyBindings);
-    
+
     value = get_value(dw, dc, dn, da, ANoAppIcon, No, useGlobalDefault);
     APPLY_VAL(value, no_appicon, ANoAppIcon);
 
-    value = get_value(dw, dc, dn, da, ACollapseAppIcons, No, useGlobalDefault);
-    APPLY_VAL(value, collapse_appicons, ACollapseAppIcons);
-    
+    value = get_value(dw, dc, dn, da, ASharedAppIcon, No, useGlobalDefault);
+    APPLY_VAL(value, shared_appicon, ASharedAppIcon);
+
     value = get_value(dw, dc, dn, da, AKeepOnTop, No, useGlobalDefault);
     APPLY_VAL(value, floating, AKeepOnTop);
 
@@ -342,13 +345,15 @@ get_generic_value(WScreen *scr, char *instance, char *class, WMPropList *option,
     WMPLSetCaseSensitive(True);
 
     if (class && instance) {
-	char *buffer = NULL;
-	buffer = wmalloc(strlen(class)+strlen(instance)+4);
-	key = WMCreatePLString(strcat(strcat(strcpy(buffer,instance),"."),class));
+        char *buffer;
+
+        buffer = wmalloc(strlen(class)+strlen(instance)+2);
+        sprintf(buffer, "%s.%s", instance, class);
+	key = WMCreatePLString(buffer);
+	wfree(buffer);
 
 	dict = WMGetFromPLDictionary(WDWindowAttributes->dictionary, key);
 	WMReleasePropList(key);
-	wfree(buffer);
 
 	if (dict) {
 	    value = WMGetFromPLDictionary(dict, option);
@@ -508,7 +513,7 @@ wDefaultChangeIcon(WScreen *scr, char *instance, char* class, char *file)
     if (instance && class) {
     	char *buffer;
         buffer = wmalloc(strlen(instance) + strlen(class) + 2);
-        strcat(strcat(strcpy(buffer, instance), "."), class);
+        sprintf(buffer, "%s.%s", instance, class);
         key = WMCreatePLString(buffer);
         wfree(buffer);
     } else if (instance) {
@@ -535,15 +540,16 @@ wDefaultChangeIcon(WScreen *scr, char *instance, char* class, char *file)
     if ((attr = WMGetFromPLDictionary(dict, key)) != NULL) {
         if (WMIsPLDictionary(attr)) {
             if (icon_value!=NULL && !same)
-                WMMergePLDictionaries(attr, icon_value);
+                WMMergePLDictionaries(attr, icon_value, False);
             else
                 WMRemoveFromPLDictionary(attr, AIcon);
         }
     } else if (icon_value!=NULL && !same) {
         WMPutInPLDictionary(dict, key, icon_value);
     }
-    if (!wPreferences.flags.noupdates)
-	WMWritePropListToFile(dict, db->path, True);
+    if (!wPreferences.flags.noupdates) {
+        UpdateDomainFile(db);
+    }
 
     WMReleasePropList(key);
     if(icon_value)
