@@ -98,11 +98,7 @@ execWindowOptionCommand(WMenu *menu, WMenuEntry *entry)
 	break;
 
      case WO_OMNIPRESENT:
-	wwin->flags.omnipresent^=1;
-	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_WORKSPACE);
-#ifdef GNOME_STUFF
-	wGNOMEUpdateClientStateHint(wwin, False);
-#endif
+	wWindowSetOmnipresent(wwin, !wwin->flags.omnipresent);
 	break;
     }
 }
