@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,   
  *  USA.
  */
 #include "wconfig.h"
@@ -46,22 +46,19 @@
 #include "actions.h"
 #include "workspace.h"
 #include "appicon.h"
-#ifdef GNOME_STUFF
-#include "gnome.h"
-#endif
 #ifdef KWM_HINTS
 #include "kwm.h"
 #endif
 
-#include <proplist.h>
+#include <WINGs/WUtil.h>
 
 
 extern WPreferences wPreferences;
 extern XContext wWinContext;
 
 
-static proplist_t dWorkspaces=NULL;
-static proplist_t dClip, dName;
+static WMPropList *dWorkspaces=NULL;
+static WMPropList *dClip, *dName;
 
 #ifdef VIRTUAL_DESKTOP
 static BOOL initVDesk = False;
@@ -73,9 +70,9 @@ make_keys()
     if (dWorkspaces!=NULL)
 	return;
 
-    dWorkspaces = PLMakeString("Workspaces");
-    dName = PLMakeString("Name");
-    dClip = PLMakeString("Clip");
+    dWorkspaces = WMCreatePLString("Workspaces");
+    dName = WMCreatePLString("Name");
+    dClip = WMCreatePLString("Clip");
 }
 
 
@@ -134,18 +131,8 @@ wWorkspaceNew(WScreen *scr)
     wspace->height = scr->scr_height;
     wspace->width = scr->scr_width;
 #endif
-#ifdef GNOME_STUFF
-	wGNOMEUpdateWorkspaceHints(scr);
-#endif
-#ifdef KWM_HINTS
-	if (!scr->flags.kwm_syncing_count) {
-	    wKWMUpdateWorkspaceCountHint(scr);
-	    wKWMUpdateWorkspaceNameHint(scr, scr->workspace_count-1);
-	}
-#ifdef not_used
-	wKWMSetUsableAreaHint(scr, scr->workspace_count-1);
-#endif
-#endif
+	WMPostNotificationName(WMNWorkspaceCreated, scr,
+			       (void*)(scr->workspace_count-1));
 	XFlush(dpy);
 
 	return scr->workspace_count-1;
@@ -220,12 +207,8 @@ wWorkspaceDelete(WScreen *scr, int workspace)
 	wMenuRealize(menu);
     }
 
-#ifdef GNOME_STUFF
-    wGNOMEUpdateWorkspaceHints(scr);
-#endif
-#ifdef KWM_HINTS
-    wKWMUpdateWorkspaceCountHint(scr);
-#endif
+    WMPostNotificationName(WMNWorkspaceDestroyed, scr,
+			   (void*)(scr->workspace_count-1));
 
     if (scr->current_workspace >= scr->workspace_count)
 	wWorkspaceChange(scr, scr->workspace_count-1);
@@ -622,12 +605,8 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
 
     showWorkspaceName(scr, workspace);
 
-#ifdef GNOME_STUFF
-    wGNOMEUpdateCurrentWorkspaceHint(scr);
-#endif
-#ifdef KWM_HINTS
-    wKWMUpdateCurrentWorkspaceHint(scr);
-#endif
+    WMPostNotificationName(WMNWorkspaceChanged, scr, (void*)workspace);
+
 /*   XSync(dpy, False); */
 }
 
@@ -919,17 +898,10 @@ wWorkspaceRename(WScreen *scr, int workspace, char *name)
 	}
     }
 
-    UpdateSwitchMenuWorkspace(scr, workspace);
-
     if (scr->clip_icon)
         wClipIconPaint(scr->clip_icon);
 
-#ifdef GNOME_STUFF
-    wGNOMEUpdateWorkspaceNamesHint(scr);
-#endif
-#ifdef KWM_HINTS
-    wKWMUpdateWorkspaceNameHint(scr, workspace);
-#endif
+    WMPostNotificationName(WMNWorkspaceNameChanged, scr, (void*)workspace);
 }
 
 
@@ -1024,72 +996,69 @@ wWorkspaceMenuUpdate(WScreen *scr, WMenu *menu)
 
 
 void
-wWorkspaceSaveState(WScreen *scr, proplist_t old_state)
+wWorkspaceSaveState(WScreen *scr, WMPropList *old_state)
 {
-    proplist_t parr, pstr;
-    proplist_t wks_state, old_wks_state;
-    proplist_t foo, bar;
+    WMPropList *parr, *pstr, *wks_state, *old_wks_state, *foo, *bar;
     int i;
 
     make_keys();
 
-    old_wks_state = PLGetDictionaryEntry(old_state, dWorkspaces);
-    parr = PLMakeArrayFromElements(NULL);
+    old_wks_state = WMGetFromPLDictionary(old_state, dWorkspaces);
+    parr = WMCreatePLArray(NULL);
     for (i=0; i < scr->workspace_count; i++) {
-        pstr = PLMakeString(scr->workspaces[i]->name);
-        wks_state = PLMakeDictionaryFromEntries(dName, pstr, NULL);
-        PLRelease(pstr);
+        pstr = WMCreatePLString(scr->workspaces[i]->name);
+        wks_state = WMCreatePLDictionary(dName, pstr, NULL);
+        WMReleasePropList(pstr);
         if (!wPreferences.flags.noclip) {
             pstr = wClipSaveWorkspaceState(scr, i);
-            PLInsertDictionaryEntry(wks_state, dClip, pstr);
-            PLRelease(pstr);
+            WMPutInPLDictionary(wks_state, dClip, pstr);
+            WMReleasePropList(pstr);
         } else if (old_wks_state!=NULL) {
-            if ((foo = PLGetArrayElement(old_wks_state, i))!=NULL) {
-                if ((bar = PLGetDictionaryEntry(foo, dClip))!=NULL) {
-                    PLInsertDictionaryEntry(wks_state, dClip, bar);
+            if ((foo = WMGetFromPLArray(old_wks_state, i))!=NULL) {
+                if ((bar = WMGetFromPLDictionary(foo, dClip))!=NULL) {
+                    WMPutInPLDictionary(wks_state, dClip, bar);
                 }
             }
         }
-        PLAppendArrayElement(parr, wks_state);
-        PLRelease(wks_state);
+        WMAddToPLArray(parr, wks_state);
+        WMReleasePropList(wks_state);
     }
-    PLInsertDictionaryEntry(scr->session_state, dWorkspaces, parr);
-    PLRelease(parr);
+    WMPutInPLDictionary(scr->session_state, dWorkspaces, parr);
+    WMReleasePropList(parr);
 }
 
 
 void
 wWorkspaceRestoreState(WScreen *scr)
 {
-    proplist_t parr, pstr, wks_state;
-    proplist_t clip_state;
+    WMPropList *parr, *pstr, *wks_state, *clip_state;
     int i, j, wscount;
 
     make_keys();
 
-    parr = PLGetDictionaryEntry(scr->session_state, dWorkspaces);
+    parr = WMGetFromPLDictionary(scr->session_state, dWorkspaces);
 
     if (!parr)
 	return;
 
     wscount = scr->workspace_count;
-    for (i=0; i < WMIN(PLGetNumberOfElements(parr), MAX_WORKSPACES); i++) {
-        wks_state = PLGetArrayElement(parr, i);
-        if (PLIsDictionary(wks_state))
-            pstr = PLGetDictionaryEntry(wks_state, dName);
+    for (i=0; i < WMIN(WMGetPropListItemCount(parr), MAX_WORKSPACES); i++) {
+        wks_state = WMGetFromPLArray(parr, i);
+        if (WMIsPLDictionary(wks_state))
+            pstr = WMGetFromPLDictionary(wks_state, dName);
         else
             pstr = wks_state;
 	if (i >= scr->workspace_count)
 	    wWorkspaceNew(scr);
 	if (scr->workspace_menu) {
 	    wfree(scr->workspace_menu->entries[i+2]->text);
-	    scr->workspace_menu->entries[i+2]->text = wstrdup(PLGetString(pstr));
+	    scr->workspace_menu->entries[i+2]->text = wstrdup(WMGetFromPLString(pstr));
 	    scr->workspace_menu->flags.realized = 0;
 	}
 	wfree(scr->workspaces[i]->name);
-        scr->workspaces[i]->name = wstrdup(PLGetString(pstr));
+        scr->workspaces[i]->name = wstrdup(WMGetFromPLString(pstr));
         if (!wPreferences.flags.noclip) {
-            clip_state = PLGetDictionaryEntry(wks_state, dClip);
+            clip_state = WMGetFromPLDictionary(wks_state, dClip);
             if (scr->workspaces[i]->clip)
                 wDockDestroy(scr->workspaces[i]->clip);
             scr->workspaces[i]->clip = wDockRestoreState(scr, clip_state,
@@ -1112,13 +1081,9 @@ wWorkspaceRestoreState(WScreen *scr)
                 }
             }
         }
-#ifdef KWM_HINTS
-	wKWMUpdateWorkspaceNameHint(scr, i);
-#endif
+
+	WMPostNotificationName(WMNWorkspaceNameChanged, scr, (void*)i);
     }
-#ifdef GNOME_STUFF
-    wGNOMEUpdateWorkspaceNamesHint(scr);
-#endif
 }
 
 
